/**********************************************************************//**
* @file		A33M11x_hal_adc.c
* @brief	Contains all functions support for adc firmware library
* 			on A33M11x
* @version	1.0
* @date		
* @author	ABOV PTS team
*
* Copyright(C) 2020, ABOV Semiconductor
* All rights reserved.
*
************************************************************************
* ABOV Disclaimer
*
*IMPORTANT NOTICE ? PLEASE READ CAREFULLY
*ABOV Semiconductor ("ABOV") reserves the right to make changes, corrections, enhancements, 
*modifications, and improvements to ABOV products and/or to this document at any time without notice. 
*ABOV does not give warranties as to the accuracy or completeness of the information included herein.
*Purchasers should obtain the latest relevant information of ABOV products before placing orders. 
*Purchasers are entirely responsible for the choice, selection, and use of ABOV products and 
*ABOV assumes no liability for application assistance or the design of purchaser products. No license, 
*express or implied, to any intellectual property rights is granted by ABOV herein. 
*ABOV disclaims all express and implied warranties and shall not be responsible or
*liable for any injuries or damages related to use of ABOV products in such unauthorized applications. 
*ABOV and the ABOV logo are trademarks of ABOV.
*All other product or service names are the property of their respective owners. 
*Information in this document supersedes and replaces the information previously
*supplied in any former versions of this document.
*2020 ABOV Semiconductor  All rights reserved
*
**********************************************************************/

/* Includes ------------------------------------------------------------------- */
#include "A33M11x_hal_adc.h"
#include "A33M11x_hal_scu.h"
#include "A33M11x_hal_libcfg.h"


/* Public Functions ----------------------------------------------------------- */
/**********************************************************************//**
 * @brief 		Initial for ADC
 * @param[in]	ADCx pointer to ADC_Type, should be:
 * 					- ADC0	:ADC0 peripheral
 * 					- ADC1	:ADC1 peripheral
 * @param[in]	ADC_ConfigStruct
 * @return 		HAL Status
 **********************************************************************/
HAL_Status_Type HAL_ADC_Init(ADC_Type *ADCx, ADC_CFG_Type *ADC_ConfigStruct)
{
	uint32_t tempreg;

	 /* Check ADC handle */
         if(ADCx == NULL)
        {
            return HAL_ERROR;
         }
		 
	//System config enable
	SYST_ACCESS_EN();
		
	// Turn on power and clock
	if(ADCx == ADC0)
	{
		SCU->PER2&=~(1<<20);
		SCU->PCER2&=~(1<<20);
		/* Set up peripheral clock for ADC0 module */
		SCU->PER2|=(1<<20);
		SCU->PCER2|=(1<<20);
	}
	else if(ADCx == ADC1)
	{
		SCU->PER2&=~(1<<21);
		SCU->PCER2&=~(1<<21);
		/* Set up peripheral clock for ADC1 module */
		SCU->PER2|=(1<<21);
		SCU->PCER2|=(1<<21);
	}
	
	//System config disable
	SYST_ACCESS_DIS();
	
	
	tempreg=0;
	switch(ADC_ConfigStruct->Mode){
		case ADC_SINGLE_MODE:
			tempreg = 0
				| (ADC_ConfigStruct->DmaOpt<<17) // DMAEN
				| ((ADC_ConfigStruct->SamplingTime & 0x1f)<<12)  // STSEL
				| (((ADC_ConfigStruct->SeqCnt-1) & 7)<<8) 		// SEQCNT
				| (1<<7)  // ADCEN
				| (((ADC_ConfigStruct->RestartEn) & 1)<<6) 		// ARST
				| (0<<4)  // ADCMOD
				| (((ADC_ConfigStruct->TrgSel) & 3)<<0)  		// TRGSRC
				;
			break;
		case ADC_BURST_MODE:
			tempreg = 0
				| (ADC_ConfigStruct->DmaOpt<<17) // DMAEN
				| ((ADC_ConfigStruct->SamplingTime & 0x1f)<<12)  // STSEL //
				| (((ADC_ConfigStruct->SeqCnt-1) & 7)<<8)  		// SEQCNT//	| ((ADC_ConfigStruct->SeqCnt+3)<<8)  // SEQCNT
				| (1<<7)  // ADCEN
				| (((ADC_ConfigStruct->RestartEn) & 1)<<6)  	// ARST
				| (1<<4)  // ADCMOD
				| (((ADC_ConfigStruct->TrgSel) & 3)<<0)  		// TRGSRC
				;
			break;
		case ADC_MULTI_MODE:
			tempreg = 0
				| (ADC_ConfigStruct->DmaOpt<<17) // DMAEN
				| ((ADC_ConfigStruct->SamplingTime & 0x1F)<<12)	// STSEL
				| (((ADC_ConfigStruct->SeqCnt-1) & 7)<<8)  		// SEQCNT//	| ((ADC_ConfigStruct->SeqCnt+3)<<8)  // SEQCNT
				| (1<<7)  // ADCEN
				| (((ADC_ConfigStruct->RestartEn) & 1)<<6)  	// ARST
				| (2<<4)  // ADCMOD
				| (((ADC_ConfigStruct->TrgSel) & 3)<<0)  		// TRGSRC
				;
		
		default:
			break;
	}
	ADCx->MR = tempreg;


	tempreg = 0
		| (0<<8)  // CKDIV  1.5Mhz * 15 = 22.5Mhz
		| (0<<7)  // ADCPD
		| (ADC_ConfigStruct->UseClk<<6)  // CLKSEL
		| (0<<5)  // CLKINVT
		;
	if (ADC_ConfigStruct->UseClk == ADC_INTERNAL_CLK) {
		tempreg |= ((ADC_ConfigStruct->InClkDiv & 0x7f)<<8)	// CKDIV  1.5Mhz * 15 =22.5Mhz,  PCLK / CLKDIV =48Mhz / 4 =  12Mhz
		;
	}
//	else if (ADC_ConfigStruct->UseClk == ADC_EXTERNAL_CLK) {
//		SCU_SetMCCRx(4, ADCx_TYPE, SCU_MCCR_CSEL_MCLK, 2);	// ACLK = MCLK/2
//	}
	
	ADCx->CCR = tempreg;
	ADCx->CSCR = 0;

	HAL_ADC_Stop(ADCx);
	return HAL_OK;
}


/**********************************************************************//**
* @brief 		Close ADC
* @param[in]	ADCx pointer to ADC_Type, should be:
* 					- ADC0	:ADC0 peripheral
* 					- ADC1	:ADC1 peripheral
* @return 		None
**********************************************************************/
HAL_Status_Type HAL_ADC_DeInit(ADC_Type *ADCx)
{
	/* Check ADC handle */
	if(ADCx == NULL)
	{
		 return HAL_ERROR;
	}

	ADCx->CR = (1<<7); // stop
	ADCx->CCR = 0;
	ADCx->IER = 0;
	ADCx->MR = 0;
	return HAL_OK;
}

/**********************************************************************//**
* @brief 		Get Result conversion from A/D data register
* @param[in]	ADCx pointer to ADC_Type, should be:
* 					- ADC0	:ADC0 peripheral
* 					- ADC1	:ADC1 peripheral
* @return 		Result of conversion
**********************************************************************/
uint16_t HAL_ADC_GetData(ADC_Type *ADCx)
{
	uint16_t adc_value;


	adc_value = (ADCx->DDR>>4);
	return ADC_DR_RESULT(adc_value);
}

/**********************************************************************//**
* @brief 		Set start mode for ADC
* @param[in]	ADCx pointer to ADC_Type, should be:
* 					- ADC0	:ADC0 peripheral
* 					- ADC1	:ADC1 peripheral
* @return 		HAL Status
*********************************************************************/
HAL_Status_Type HAL_ADC_Start(ADC_Type *ADCx)
{
      	/* Check ADC handle */
	if(ADCx == NULL)
	{
		return HAL_ERROR;
	}
	
	ADCx->CR =ADC_CR_START_MASK;
	
//	HAL_ADC_GetData(ADCx); // dummy data read
	return HAL_OK;
}

/**********************************************************************//**
* @brief 		Set stop mode for ADC
* @param[in]	ADCx pointer to ADC_Type, should be:
* 					- ADC0	:ADC0 peripheral
* 					- ADC1	:ADC1 peripheral
* @return 		HAL Status
*********************************************************************/
HAL_Status_Type HAL_ADC_Stop(ADC_Type *ADCx)
{
      	/* Check ADC handle */
	if(ADCx == NULL)
	{
		return HAL_ERROR;
	}
	ADCx->CR =ADC_CR_STOP_MASK;
	return HAL_OK;
}

/**********************************************************************//**
* @brief 		Set AD conversion in power mode
* @param[in]	ADCx pointer to ADC_Type, should be:
* 					- ADC0	:ADC0 peripheral
* 					- ADC1	:ADC1 peripheral	
* @param[in]	NewState
* 					- DISABLE : AD converter is normal mode
* 					- ENABLE: AD Converter is in power down mode
* @return 		None
**********************************************************************/
HAL_Status_Type HAL_ADC_EnterPowerdownMode(ADC_Type *ADCx, FunctionalState NewState)
{
       	/* Check ADC handle */
	if(ADCx == NULL)
	{
		return HAL_ERROR;
	}
	ADCx->CCR &= ~ADC_CCR_PDN;
	if (NewState){
		ADCx->CCR |= ADC_CCR_PDN;
	}
	return HAL_OK;
}

/**********************************************************************//**
* @brief 		ADC interrupt configuration
* @param[in]	ADCx pointer to ADC_Type, should be:
* 					- ADC0	:ADC0 peripheral
* 					- ADC1	:ADC1 peripheral
* @param[in]	IntType: type of interrupt, should be:
* 					- ADC_INTEN_DMA
* 					- ADC_INTEN_TRIGGER
* 					- ADC_INTEN_SEQ
* 					- ADC_INTEN_SINGLE
* @param[in]	NewState:
* 					- SET : enable ADC interrupt
* 					- RESET: disable ADC interrupt
* @return 		HAL Status
**********************************************************************/
HAL_Status_Type HAL_ADC_ConfigInterrupt(ADC_Type *ADCx, uint8_t IntType, FunctionalState NewState)
{
       	/* Check ADC handle */
	if(ADCx == NULL)
	{
		return HAL_ERROR;
	}
	ADCx->IER &= ~IntType;
	if (NewState){
		ADCx->IER |= IntType;
	}
	return HAL_OK;
}

/**********************************************************************//**
* @brief 		Select ADC channel number
* @param[in]	ADCx pointer to ADC_Type, should be:
* 					- ADC0	:ADC0 peripheral
* 					- ADC1	:ADC1 peripheral
* @param[in]	Channel channel number, should be : 
* 					- 0 ~ 23
* @return 		HAL Status
**********************************************************************/
HAL_Status_Type HAL_ADC_ChannelSel(ADC_Type *ADCx, uint32_t Channel)
{
        /* Check ADC handle */
	if(ADCx == NULL)
	{
		return HAL_ERROR;
	}
	ADCx->SCSR1 = Channel;
	return HAL_OK;
}

/**********************************************************************//**
* @brief 		Select ADC channel number
* @param[in]	ADCx pointer to ADC_Type, should be:
* 					- ADC0	:ADC0 peripheral
* 					- ADC1	:ADC1 peripheral
* @param[in]	Channel channel number, should be :
* 					- 0 ~ 23
* @return 		HAL Status
**********************************************************************/
HAL_Status_Type HAL_ADC_ChannelSel_1(ADC_Type *ADCx, uint32_t Channel)
{
       	/* Check ADC handle */
	if(ADCx == NULL)
	{
		return HAL_ERROR;
	}
	ADCx->SCSR1 = Channel;
	return HAL_OK;
}

/**********************************************************************//**
* @brief 		Select ADC channel number
* @param[in]	ADCx pointer to ADC_Type, should be:
* 					- ADC0	:ADC0 peripheral
* 					- ADC1	:ADC1 peripheral
* @param[in]	Channel channel number, should be : 
* 					- 0 ~ 23
* @return 		HAL Status
**********************************************************************/
HAL_Status_Type HAL_ADC_ChannelSel_2(ADC_Type *ADCx, uint32_t Channel)
{
     	/* Check ADC handle */
	if(ADCx == NULL)
	{
		return HAL_ERROR;
	}
	ADCx->SCSR2 = Channel;
	return HAL_OK;
}

/**********************************************************************//**
* @brief 		Select ADC trigger number
* @param[in]	ADCx pointer to ADC_Type, should be:
* 					- ADC0	:ADC0 peripheral
* 					- ADC1	:ADC1 peripheral
* @param[in]	Channel channel number
* @return 		HAL Status
**********************************************************************/
HAL_Status_Type HAL_ADC_TriggerlSel(ADC_Type *ADCx, uint32_t Trigger)
{
      	/* Check ADC handle */
	if(ADCx == NULL)
	{
		return HAL_ERROR;
	}
	ADCx->TRG = Trigger;
	return HAL_OK;
}

/**********************************************************************//**
* @brief 		Get ADC channel status
* @param[in]	ADCx pointer to ADC_Type, should be:
* 					- ADC0	:ADC0 peripheral
* 					- ADC1	:ADC1 peripheral
* @return 		ADC status register
**********************************************************************/
uint32_t HAL_ADC_GetStatus(ADC_Type *ADCx)
{

	return (ADCx->SR);
}

/**********************************************************************//**
* @brief 		Clear ADC Chanel status
* @param[in]	ADCx pointer to ADC_Type, should be:
* 					- ADC0	:ADC0 peripheral
* 					- ADC1	:ADC1 peripheral
* @param[in]	status ADCx Interrupt Status Clear Bit, should be:
* 					- ADC_STAT_COMPIFLG	  ((1UL<<8))
* 					- ADC_STAT_DMA		  ((1UL<<4))
* 					- ADC_STAT_TRIGGER	  ((1UL<<3))
* 					- ADC_STAT_SEQ		  ((1UL<<2))
* 					- ADC_STAT_SINGLE	  ((1UL<<0))
* @return 		HAL Status
**********************************************************************/
HAL_Status_Type HAL_ADC_ClearStatus(ADC_Type *ADCx, uint32_t status)
{
      	/* Check ADC handle */
	if(ADCx == NULL)
	{
		return HAL_ERROR;
	}
	ADCx->SR=status;
	return HAL_OK;
}


