/**********************************************************************//**
* @file		A33M11X_hal_afe.c
* @brief	Contains all functions support for AFE firmware library on A33M11x
* @version	1.0
* @date		
* @author	ABOV PTS team
*
* Copyright(C) 2020, ABOV Semiconductor
* All rights reserved.
*
************************************************************************
* ABOV Disclaimer
*
*IMPORTANT NOTICE ? PLEASE READ CAREFULLY
*ABOV Semiconductor ("ABOV") reserves the right to make changes, corrections, enhancements, 
*modifications, and improvements to ABOV products and/or to this document at any time without notice. 
*ABOV does not give warranties as to the accuracy or completeness of the information included herein.
*Purchasers should obtain the latest relevant information of ABOV products before placing orders. 
*Purchasers are entirely responsible for the choice, selection, and use of ABOV products and 
*ABOV assumes no liability for application assistance or the design of purchaser products. No license, 
*express or implied, to any intellectual property rights is granted by ABOV herein. 
*ABOV disclaims all express and implied warranties and shall not be responsible or
*liable for any injuries or damages related to use of ABOV products in such unauthorized applications. 
*ABOV and the ABOV logo are trademarks of ABOV.
*All other product or service names are the property of their respective owners. 
*Information in this document supersedes and replaces the information previously
*supplied in any former versions of this document.
*2020 ABOV Semiconductor  All rights reserved
*
**********************************************************************/

/* Includes ------------------------------------------------------------------- */
#include "A33M11x_hal_afe.h"
#include "A33M11x_hal_scu.h"

/***********************************************************************************************//**
 * @brief		Get Comparator Status
 * @param[in]	None
 * @return	STAT register value
 ***********************************************************************************************/
uint32_t HAL_AFE_COMP_GetStatus(void)
{
	return AFE->CMP_SR;
}

/***********************************************************************************************//**
 * @brief		Clear Comparator status 
* @param[in]	Type		comparator Clear interrupt Status select, should be :
						- AFE_CMPSR_CMP0INTF
						- AFE_CMPSR_CMP1INTF
						- AFE_CMPSR_CMP2INTF
						- AFE_CMPSR_CMP3INTF
 * @return	None
 ***********************************************************************************************/
void HAL_AFE_COMP_ClearStatus(uint32_t Type)
{
	AFE->CMP_SR |= Type;
}

/***********************************************************************************************//**
 * @brief		Clear All Comparator status
 * @param[in]	None
 * @return	None
 ***********************************************************************************************/
void HAL_AFE_COMP_ClearStatusAll(void)
{
	AFE->CMP_SR |= AFE_CMPSR_CMPINTF_BITMASK;
}

/***********************************************************************************************//**
* @brief		Configure opamp&comparator peripheral and clock reference/input voltage
* @param[in]	NewState		should be :
* 						- DISABLE= 0
* 						- ENABLE	= 1							
* @return	None
 ***********************************************************************************************/
void HAL_AFE_ClockInit(FunctionalState NewState)
{
	//AFE peripheral & Clock Enable
	SYST_ACCESS_EN();
	
	SCU->PER2 &= ~(1<<24);
	SCU->PCER2 &= ~(1<<24); 
	
	if(NewState == ENABLE)
	{
		SCU->PER2 |= (1<<24);
		SCU->PCER2 |= (1<<24); 
	}
	
	SYST_ACCESS_DIS();
}

/***********************************************************************************************//**
* @brief		Enable/disable Comparator operate
* @param[in]	cmpnum		select comparator number, should be :
*						- COMP0 : 0, * - COMP1 : 1
*						- COMP2 : 2, * - COMP3 : 3
* @param[in]	NewState	 	should be :
* 						- DISABLE= 0
* 						- ENABLE	= 1
 * @return	None
 ***********************************************************************************************/
HAL_Status_Type HAL_AFE_COMP_Cmd(uint8_t afenum,  FunctionalState NewState)
{
	/* Check AFE handler */
	if(afenum>3)
	{
		return HAL_ERROR;
	}
	
	if(afenum == 0){
		AFE->AFE0_CR &= ~(AFE_CR_CMPEN);
		if(NewState == ENABLE)
		{
			AFE->AFE0_CR |= AFE_CR_CMPEN;
		}
	}
	else if(afenum == 1){
		AFE->AFE1_CR &= ~(AFE_CR_CMPEN);
		if(NewState == ENABLE)
		{
			AFE->AFE1_CR |= AFE_CR_CMPEN;
		}
	}
	else if(afenum == 2){
		AFE->AFE2_CR &= ~(AFE_CR_CMPEN);
		if(NewState == ENABLE)
		{
			AFE->AFE2_CR |= AFE_CR_CMPEN;
		}
	}
	else if(afenum == 3){
		AFE->AFE3_CR &= ~(AFE_CR_CMPEN);
		if(NewState == ENABLE)
		{
			AFE->AFE3_CR |= AFE_CR_CMPEN;
		}
	}
	return HAL_OK;	
}

/***********************************************************************************************//**
 * @brief		Set Comparator Debounce 
* @param[in]	cmpnum	select comparator number, should be :
*						- COMP0 : 0, * - COMP1 : 1
*						- COMP2 : 2, * - COMP3 : 3
 * @param[in]	dbnccounter	Debounce time base counter, should be in range from 0 to 0xF
 * @return		None
 ***********************************************************************************************/
HAL_Status_Type HAL_AFE_COMP_DebounceFilter(uint8_t afenum, uint32_t dbnccounter)
{
		/* Check AFE handler */
	if(afenum>3)
	{
		return HAL_ERROR;
	}
	
	if(afenum == 0)
	{	
		AFE->CMP_DBR &= ~(AFE_CMPDBR_C0FLTSEL|AFE_CMPDBR_FLTSEL_BITMASK);
		AFE->CMP_DBR |= (dbnccounter << 0);
	}
	else if(afenum == 1)
	{	
		AFE->CMP_DBR &= ~(AFE_CMPDBR_C1FLTSEL|AFE_CMPDBR_FLTSEL_BITMASK);
		AFE->CMP_DBR |= (dbnccounter << 4);
	}
	else if(afenum == 2)
	{	
		AFE->CMP_DBR &= ~(AFE_CMPDBR_C2FLTSEL|AFE_CMPDBR_FLTSEL_BITMASK);
		AFE->CMP_DBR |= (dbnccounter << 8);
	}
	else if(afenum == 3)
	{	
		AFE->CMP_DBR &= ~(AFE_CMPDBR_C3FLTSEL|AFE_CMPDBR_FLTSEL_BITMASK);
		AFE->CMP_DBR |= (dbnccounter << 12);
	}
	return HAL_OK;
}

/***********************************************************************************************//**
* @brief		Enable/disable Comparator Interrupt
* @param[in]	cmpnum		select comparator number, should be :
*						- COMP0 : 0, * - COMP1 : 1
*						- COMP2 : 2, * - COMP3 : 3
* @param[in]	NewState	 	should be :
* 						- DISABLE = 0
* 						- ENABLE	= 1
* @return	None
 ***********************************************************************************************/
HAL_Status_Type HAL_AFE_COMP_IntCmd(uint8_t afenum, FunctionalState NewState)
{
	/* Check AFE handler */
	if(afenum>3)
	{
		return HAL_ERROR;
	}
	
	if(afenum == 0){
		AFE->AFE0_CR &= ~(AFE_CR_CMPIEN);
		if(NewState == ENABLE)
		{
			AFE->AFE0_CR |= AFE_CR_CMPIEN;
		}
	}
	else if(afenum == 1){
		AFE->AFE1_CR &= ~(AFE_CR_CMPIEN);
		if(NewState == ENABLE)
		{
			AFE->AFE1_CR |= AFE_CR_CMPIEN;
		}
	}
	else if(afenum == 2){
		AFE->AFE2_CR &= ~(AFE_CR_CMPIEN);
		if(NewState == ENABLE)
		{
			AFE->AFE2_CR |= AFE_CR_CMPIEN;
		}
	}
	else if(afenum == 3){
		AFE->AFE3_CR &= ~(AFE_CR_CMPIEN);
		if(NewState == ENABLE)
		{
			AFE->AFE3_CR |= AFE_CR_CMPIEN;
		}
	}
	return HAL_OK;
}

/***********************************************************************************************//**
* @brief		Configure comparator interrupt 
* @param[in]	cmpnum		select comparator number, should be :
*						- COMP0 : 0, * - COMP1 : 1
*						- COMP2 : 2, * - COMP3 : 3
* @param[in]	ipol			Comparator interrupt polarity(level mode), should be :
						- INTPOL_LOW		= 0 					
						- INTPOL_HIGH	= 1
* @param[in]	mode		Comparator Interrupt mode select, should be :   		
						- INTTYPE_NOTUSE	= 0
						- INTTYPE_LEVEL	= 1		     			
						- INTTYPE_EDGE	= 2		     		
						- INTTYPE_BOTH_EDGE	= 3
 * @return	None
 ***********************************************************************************************/
HAL_Status_Type HAL_AFE_COMP_IntConfigure(uint8_t afenum, uint32_t ipol, uint32_t mode)
{
	/* Check AFE handler */
	if(afenum>3)
	{
		return HAL_ERROR;
	}
	
	if(afenum==0)
	{
		AFE->CMP_ICR &= ~(AFE_CMPCR_INTTYPE0_BITMASK |AFE_CMPCR_INTPOL0_BITMASK);
		AFE->CMP_ICR |= ipol<<3;
		AFE->CMP_ICR |= mode<<0;
	}
	else	if(afenum==1)
	{
		AFE->CMP_ICR &= ~(AFE_CMPCR_INTTYPE1_BITMASK |AFE_CMPCR_INTPOL1_BITMASK);
		AFE->CMP_ICR |= ipol<<7;
		AFE->CMP_ICR |= mode<<4;
	}
	else	if(afenum==2)
	{
		AFE->CMP_ICR &= ~(AFE_CMPCR_INTTYPE1_BITMASK |AFE_CMPCR_INTPOL1_BITMASK);
		AFE->CMP_ICR |= ipol<<11;
		AFE->CMP_ICR |= mode<<8;
	}
	else	if(afenum==3)
	{
		AFE->CMP_ICR &= ~(AFE_CMPCR_INTTYPE3_BITMASK |AFE_CMPCR_INTPOL3_BITMASK);
		AFE->CMP_ICR |= ipol<<15;
		AFE->CMP_ICR |= mode<<12;
	}
	return HAL_OK;
}


/***********************************************************************************************//**
* @brief		Configure comparator output 
* @param[in]	cmpnum		select comparator number, should be :
*						- COMP0 : 0, * - COMP1 : 1
*						- COMP2 : 2, * - COMP3 : 3
* @param[in]	outpol			Comparator output polarity, should be :
						- OUTPOL_NONE		= 0 					
						- OUTPOL_REVERSE	= 1
 * @return	None
 ***********************************************************************************************/
HAL_Status_Type HAL_AFE_COMP_OutputConfigure(uint8_t afenum, uint32_t outpol)
{
	/* Check AFE handler */
	if(afenum>3)
	{
		return HAL_ERROR;
	}
	
	if(afenum==0)
	{
		AFE->AFE0_CR &= ~(AFE_CR_CMPOSEL_MASK);
		AFE->AFE0_CR |= outpol<<15;
	}
	else	if(afenum==1)
	{
		AFE->AFE1_CR &= ~(AFE_CR_CMPOSEL_MASK);
		AFE->AFE1_CR |= outpol<<15;
	}
	else	if(afenum==2)
	{
		AFE->AFE2_CR &= ~(AFE_CR_CMPOSEL_MASK);
		AFE->AFE2_CR |= outpol<<15;
	}
	else	if(afenum==3)
	{
		AFE->AFE3_CR &= ~(AFE_CR_CMPOSEL_MASK);
		AFE->AFE3_CR |= outpol<<15;
	}
	return HAL_OK;
}


/***********************************************************************************************//**
* @brief		Enable/disable OPAMP Operation
* @param[in]	opanum		select afe number, should be :
*						- OPAMP0 : 0, * - OPAMP1 : 1
*						- OPAMP2 : 2, * - OPAMP3 : 3
* @param[in]	NewState	 	should be :
* 						- DISABLE = 0
* 						- ENABLE	= 1
* @return	None
 ***********************************************************************************************/
HAL_Status_Type HAL_AFE_OPAMP_UGAIN_Cmd(uint8_t afenum, FunctionalState NewState)
{
	/* Check AFE handler */
	if(afenum>3)
	{
		return HAL_ERROR;
	}
	
	if(afenum == 0){
		AFE->AFE0_CR &= ~(AFE_CR_UGAINEN_MASK);
		if(NewState == ENABLE)
		{
			AFE->AFE0_CR |= AFE_CR_UGAINEN;
		}
	}
	else if(afenum == 1){
		AFE->AFE1_CR &= ~(AFE_CR_UGAINEN_MASK);
		if(NewState == ENABLE)
		{
			AFE->AFE1_CR |= AFE_CR_UGAINEN;
		}
	}
	else if(afenum == 2){
		AFE->AFE2_CR &= ~(AFE_CR_UGAINEN_MASK);
		if(NewState == ENABLE)
		{
			AFE->AFE2_CR |= AFE_CR_UGAINEN;
		}
	}
	else if(afenum == 3){
		AFE->AFE3_CR &= ~(AFE_CR_UGAINEN_MASK);
		if(NewState == ENABLE)
		{
			AFE->AFE3_CR |= AFE_CR_UGAINEN;
		}
	}
	return HAL_OK;
}

/***********************************************************************************************//**
* @brief		Enable/disable OPAMP Operation
* @param[in]	opanum		select afe number, should be :
*						- OPAMP0 : 0, * - OPAMP1 : 1
*						- OPAMP2 : 2, * - OPAMP3 : 3
* @param[in]	NewState	 	should be :
* 						- DISABLE = 0
* 						- ENABLE	= 1
* @return	None
 ***********************************************************************************************/
HAL_Status_Type HAL_AFE_OPAMP_Cmd(uint8_t afenum, FunctionalState NewState)
{
	/* Check AFE handler */
	if(afenum>3)
	{
		return HAL_ERROR;
	}
	
	if(afenum == 0){
		AFE->AFE0_CR &= ~(AFE_CR_OPAEN);
		if(NewState == ENABLE)
		{
			AFE->AFE0_CR |= AFE_CR_OPAEN;
		}
	}
	else if(afenum == 1){
		AFE->AFE1_CR &= ~(AFE_CR_OPAEN);
		if(NewState == ENABLE)
		{
			AFE->AFE1_CR |= AFE_CR_OPAEN;
		}
	}
	else if(afenum == 2){
		AFE->AFE2_CR &= ~(AFE_CR_OPAEN);
		if(NewState == ENABLE)
		{
			AFE->AFE2_CR |= AFE_CR_OPAEN;
		}
	}
	else if(afenum == 3){
		AFE->AFE3_CR &= ~(AFE_CR_OPAEN);
		if(NewState == ENABLE)
		{
			AFE->AFE3_CR |= AFE_CR_OPAEN;
		}
	}
	return HAL_OK;
}

/***********************************************************************************************//**
* @brief		OPAMP Cuerrent Setting
* @param[in]	opanum		select afe number, should be :
*						- OPAMP0 : 0, * - OPAMP1 : 1
*						- OPAMP2 : 2, * - OPAMP3 : 3
* @param[in]	ampisel	 	should be :
* 						- AMPISEL_0PERCENT
* 						- AMPISEL_17PERCENT
* 						- AMPISEL_21PERCENT
* 						- AMPISEL_12PERCENT
* @return	None
 ***********************************************************************************************/
HAL_Status_Type HAL_AFE_OPAMP_CurrentSet(uint8_t afenum,  uint8_t ampisel)
{
	/* Check AFE handler */
	if(afenum>3)
	{
		return HAL_ERROR;
	}
	
	if(afenum == 0){
		AFE->AFE0_CR &= ~(AFE_CR_AMPISEL_MASK);
		AFE->AFE0_CR |= ampisel<<16;
	}
	else if(afenum == 1){
		AFE->AFE1_CR &= ~(AFE_CR_AMPISEL_MASK);
		AFE->AFE1_CR |= ampisel<<16;
	}
	else if(afenum == 2){
		AFE->AFE2_CR &= ~(AFE_CR_AMPISEL_MASK);
		AFE->AFE2_CR |= ampisel<<16;
	}
	else if(afenum == 3){
		AFE->AFE3_CR &= ~(AFE_CR_AMPISEL_MASK);
		AFE->AFE3_CR |= ampisel<<16;
	}
	return HAL_OK;
}

/* --------------------------------- End Of File ------------------------------ */

