/**********************************************************************//**
* @file		A33M11x_hal_dma.c
* @brief	Contains all functions support for dma firmware library
* 			on A33M11x
* @version	1.0
* @date		
* @author	ABOV PTS team
*
* Copyright(C) 2020, ABOV Semiconductor
* All rights reserved.
*
************************************************************************
* ABOV Disclaimer
*
*IMPORTANT NOTICE ? PLEASE READ CAREFULLY
*ABOV Semiconductor ("ABOV") reserves the right to make changes, corrections, enhancements, 
*modifications, and improvements to ABOV products and/or to this document at any time without notice. 
*ABOV does not give warranties as to the accuracy or completeness of the information included herein.
*Purchasers should obtain the latest relevant information of ABOV products before placing orders. 
*Purchasers are entirely responsible for the choice, selection, and use of ABOV products and 
*ABOV assumes no liability for application assistance or the design of purchaser products. No license, 
*express or implied, to any intellectual property rights is granted by ABOV herein. 
*ABOV disclaims all express and implied warranties and shall not be responsible or
*liable for any injuries or damages related to use of ABOV products in such unauthorized applications. 
*ABOV and the ABOV logo are trademarks of ABOV.
*All other product or service names are the property of their respective owners. 
*Information in this document supersedes and replaces the information previously
*supplied in any former versions of this document.
*2020 ABOV Semiconductor  All rights reserved
*
**********************************************************************/

/* Includes ------------------------------------------------------------------- */
#include "A33M11x_hal_scu.h"
#include "A33M11x_hal_dma.h"
#include "A33M11x_hal_libcfg.h"




/* Public Functions ----------------------------------------------------------- */
/*********************************************************************//**
 * @brief		Initializes the DMA peripheral 
 * @param[in]	None
 * @return 		HAL Status
 *********************************************************************/
HAL_Status_Type HAL_DMAC_Init(void)
{
	SYST_ACCESS_EN();
	
	/* to be defined Enable DMA and clock */
	// enabled default on reset
	SCU->PER1&=~(1<<4); // (1<<4) DMA peri enable 
	SCU->PCER1&=~(1<<4); // (1<<4) DMA peri clock enable 
	
	SCU->PER1|=(1<<4); // (1<<4) DMA peri enable 
	SCU->PCER1|=(1<<4); // (1<<4) DMA peri clock enable 
	
	SYST_ACCESS_DIS();
	return HAL_OK;
}

/*********************************************************************//**
* @brief      	Initialize DMAC(Direct Memory Access Controller) peripheral
* @param    	None
* @return     	HAL_Status
*********************************************************************/
HAL_Status_Type HAL_DMAC_DeInit(void)
{
	SCU->PER1&=~(1<<4); // (1<<4) DMA peri enable 
	SCU->PCER1&=~(1<<4); // (1<<4) DMA peri clock enable 
	
	return HAL_OK;
}

/*********************************************************************//**
 * @brief 		Setup DMAC channel peripheral according to the specified
 *              parameters in the GPDMAChannelConfig.
 * @param[in]	DMACx	Pointer to selected DMAC peripheral, should be:
 * 					- DMAx	:DMA0~7 peripheral
 * @param[in]	DMACChannelConfig Pointer to a DMAC_CH_CFG_Type structure
 * 				that contains the configuration information for the specified
 * 				channel peripheral.
 * @return		HAL Status
 *********************************************************************/
HAL_Status_Type HAL_DMAC_Setup(DMA_Type *DMAx, DMA_CFG_Type *DMACChConfig)
{
	DMAx->CR = 0
	| ((DMACChConfig->transcnt&0xFFF)<<16)
	| ((DMACChConfig->perisel&0x1F)<<8)
	| ((DMACChConfig->bussize&(0x3<<2)))
	| ((DMACChConfig->dirsel&(1<<1)))
	;
	DMAx->PAR = DMACChConfig->periaddr;	
	DMAx->MAR = DMACChConfig->memaddr;
	return HAL_OK;
}


/**********************************************************************//**
 * @brief		Enable/Disable DMA channel
 * @param[in]	DMACx	Pointer to selected DMAC peripheral, should be:
 * 					- DMAx	:DMAC0~7 peripheral
 * @param[in]	NewState	New State of this command, should be:
 * 					- ENABLE.
 * 					- DISABLE.
 * @return		HAL Status
 **********************************************************************/
HAL_Status_Type HAL_DMAC_ChCmd(DMA_Type *DMAx, FunctionalState NewState)
{
	DMAx->SR |= (1<<4);
	if (NewState == ENABLE) {
		DMAx->SR |=  (1<<0);
	} else {
		DMAx->SR &= ~(1<<0);
	}
	return HAL_OK;
}



/**********************************************************************//**
 * @brief		Check if corresponding channel does have an active interrupt
 * 				request or not
 * @param[in]	DMAx	Pointer to selected DMAC peripheral, should be:
 * 					- DMAx	:DMAC0~7 peripheral
 * @return		FlagStatus	status of DMA channel interrupt after masking
 * 				Should be:
 * 					- SET	: all data is transferred 
 * 					- RESET	:data to be transferred is existing 
 **********************************************************************/

FlagStatus HAL_DMAC_GetStatus(DMA_Type* DMAx)
{
	if ((DMAx->SR & DMA_SR_EOT)==DMA_SR_EOT){	
		return SET;
	}
	else {
		return RESET;
	}
}

/* --------------------------------- End Of File ------------------------------ */



