/**********************************************************************//**
* @file		A33M11x_hal_frt.c
* @brief	Contains all functions support for FRT(Free Running TImer) firmware library on A33M11x
* @version	1.0
* @date		
* @author	ABOV PTS team
*
* Copyright(C) 2020, ABOV Semiconductor
* All rights reserved.
*
************************************************************************
* ABOV Disclaimer
*
*IMPORTANT NOTICE ? PLEASE READ CAREFULLY
*ABOV Semiconductor ("ABOV") reserves the right to make changes, corrections, enhancements, 
*modifications, and improvements to ABOV products and/or to this document at any time without notice. 
*ABOV does not give warranties as to the accuracy or completeness of the information included herein.
*Purchasers should obtain the latest relevant information of ABOV products before placing orders. 
*Purchasers are entirely responsible for the choice, selection, and use of ABOV products and 
*ABOV assumes no liability for application assistance or the design of purchaser products. No license, 
*express or implied, to any intellectual property rights is granted by ABOV herein. 
*ABOV disclaims all express and implied warranties and shall not be responsible or
*liable for any injuries or damages related to use of ABOV products in such unauthorized applications. 
*ABOV and the ABOV logo are trademarks of ABOV.
*All other product or service names are the property of their respective owners. 
*Information in this document supersedes and replaces the information previously
*supplied in any former versions of this document.
*2020 ABOV Semiconductor  All rights reserved
*
**********************************************************************/
/* Includes ------------------------------------------------------------------- */
#include "A33M11x_hal_frt.h"
#include "A33M11x_hal_libcfg.h"
#include "A33M11x_hal_scu.h"
#include "A33M11x_hal_debug_frmwrk.h"	

/* Public Functions ----------------------------------------------------------- */
/*********************************************************************//**
 * @brief		Initialization register related FRT function at SCU
 * @param[in]	MODE_v : Mode Selection Bit
			- FRT_CTRL_MODE_FREE_RUN
			- FRT_CTRL_MODE_MATCH
 * @return 		None
 **********************************************************************/
HAL_Status_Type HAL_FRT_Init(uint8_t MODE_v)
{
	 /* Check FRT  handle */
	 if((MODE_v != FRT_CTRL_MODE_FREE_RUN) && (MODE_v != FRT_CTRL_MODE_MATCH))
	{
			return HAL_ERROR;
	}
		
	SYST_ACCESS_EN();
	
	SCU->PER1 &= ~(1 << 6); 
	SCU->PCER1 &= ~(1 << 6);
	
	SCU->PER1 |= (1 << 6);			// FRT0 Enable
	SCU->PCER1 |= (1 << 6);			// FRT0 Clock Enable
	
	SYST_ACCESS_DIS();

	HAL_FRT_ModeSelection(MODE_v);
	
	 /* Register Reset */
	 HAL_FRT_ClearCounter();
	 
	 return HAL_OK;	 
	
}

/*********************************************************************//**
 * @brief		Set Clock Source Type of FRT
 * @param[in]	Clock Source:
 *					- SCU_MCCR_CSEL_LSI 
 *					- SCU_MCCR_CSEL_MCLK 
 *					- SCU_MCCR_CSEL_HSI 
 *					- SCU_MCCR_CSEL_HSE 
 *					- SCU_MCCR_CSEL_PLL 
 * @param[in]	Clock Divider: 
 *					- 0x0~0xFF
 * @return 		None
 **********************************************************************/
void HAL_FRT_ClockSource(uint8_t ClkSource, uint8_t ClkDivider)
{		 
	SYST_ACCESS_EN();
	
	SCU->MCCR6 &= ~(0x07<<8);
	SCU->MCCR6 &= ~(0xFF<<0);

	SCU->MCCR6 |= (((ClkSource&0x07)<<8) | ((ClkDivider&0xFF)<<0));
	
	SYST_ACCESS_DIS();
}

/*********************************************************************//**
 * @brief		Get Clock Source Type of FRT
 * @param[in]	None 
 * @return 		Clock Source Type
 **********************************************************************/
uint8_t HAL_FRT_GetClockSource(void)
{
	uint8_t ClkSource;
	
	ClkSource = ((SCU->MCCR6&(0x07<<8))>>8);
	
	return ClkSource;
}


/*********************************************************************//**
 * @brief		Set FRT Operation 
 * @param[in]	isset	Enable or Disable
 *					- 1		:Enable 
 *					- 0		:Disable
 * @return 		None
 **********************************************************************/
void HAL_FRT_Run(uint8_t isset)
{
	if(isset)
	{
		FRT->CTRL |= 0x01;		// FRT Enable
//		_DBG("FRT is Enabled.\r\n");
	}
	else
	{
		FRT->CTRL &= ~0x01;		// FRT Disable				
//		_DBG("FRT is Disabled.\r\n");
	}
}

/*********************************************************************//**
 * @brief		Set FRT Match Interrupt
 * @param[in]	isset	Enable or Disable
 *					- 1		:Enable 
 *					- 0		:Disable
 * @return 		None
 **********************************************************************/
void HAL_FRT_MatchInterrupt(uint8_t isset)
{
	if(isset)
	{
		FRT->CTRL |= (1<<8);		// FRT Match Interrupt Enable					
//		_DBG("FRT Interrupt is Enabled.\r\n");		
	}
	else
	{
		FRT->CTRL &= ~(1<<8);		// FRT Match Interrupt Disable		
	}
}

/*********************************************************************//**
 * @brief		Set FRT OVERFLOW Interrupt Mode
 * @param[in]	isset	Enable or Disable
 *					- 1		:Enable 
 *					- 0		:Disable
 * @return 		HAL Status
 **********************************************************************/
void HAL_FRT_OverflowInterrupt(uint8_t isset)
{
	if(isset)		
	{
		FRT->CTRL |= (1<<9);		// FRT Overflow Interrupt Enable
//		_DBG("FRT Overflow Interrupt is Enabled.\r\n");
	}
	else
	{
		FRT->CTRL &= ~(1<<9);		// FRT Overflow Interrupt Disable
	}
}


/********************************************************************//**
 * @brief		Clear Status of FRT
 * @param[in]	StatusType	Status Type
 *					- FRT_STAT_OVFI
 *					- FRT_STAT_MATCHI
 * @return 		HAL Status
 **********************************************************************/
void HAL_FRT_ClearStatus(uint16_t StatusType)
{
	FRT->STAT = StatusType;
}

/*********************************************************************//**
 * @brief		Get Status of FRT
 * @param[in]	None
 * @return 		8-bit Status
 **********************************************************************/
uint16_t HAL_FRT_GetStatus(void)
{	
	return FRT->STAT;			// Get Status Register Value
}


/*********************************************************************//**
 * @brief		Get Counter Value of FRT
 * @param[in]	None
 * @return 		32-bit Count Value (uint32_t)
 **********************************************************************/
uint32_t HAL_FRT_GetCounterVal(void)
{
	uint32_t CountValue = 0;

	CountValue = FRT->CNT;				// Get FRTx Counter Value

	return CountValue;
}

/*********************************************************************//**
 * @brief		Clear Counter Value of FRT
 * @param[in]	None
 * @return 		HAL Status
 **********************************************************************/
void HAL_FRT_ClearCounter(void)
{
	uint32_t regtemp;
	
	regtemp = (0x00); 
	FRT->CNT = regtemp;			// Clear FRTx Counter Register Value
//	_DBG("FRTx Counter Value is Cleard.\r\n");
}

/*********************************************************************//**
 * @brief		Set Match Counter Value of FRT
 * @param[in]	MCNT_v	FRT Counter Value Bit
 *
 * @return 		HAL Status
 **********************************************************************/
void HAL_FRT_SetMatchCounter(uint32_t MCNT_v)
{
	uint32_t	regtemp;

	regtemp = MCNT_v;
	FRT->MCNT = regtemp;
}

/*********************************************************************//**
 * @brief		Get Match Counter Value of FRT
 * @param[in]	None
 * @return 		32-bit Match Counter Value
 **********************************************************************/
uint32_t HAL_FRT_GetMatchCounter(void)
{
	uint32_t MatchCounterValue;
	
	MatchCounterValue = FRT->MCNT;
	
	return MatchCounterValue;
}

/*********************************************************************//**
 * @brief		Mode Selection of FRT
 * @param[in]	MODE_v	FRT Mode Selection Bit
 * 					- FRT_CTRL_MODE_FREE_RUN	(0<<1)
 * 					- FRT_CTRL_MODE_MATCH		(1<<1)	
 * @return 		HAL Status
 **********************************************************************/
void HAL_FRT_ModeSelection(uint8_t MODE_v)
{
	uint32_t	regtemp;
	
	regtemp = FRT->CTRL;
	regtemp &= ~(0x01<<1);
	regtemp |= (MODE_v&(1<<1));
		
	FRT->CTRL = regtemp;
}




/* --------------------------------- End Of File ------------------------------ */

