/**********************************************************************//**
* @file		A33M11x_hal_mpwm.c
* @brief	Contains all functions support for MPWM firmware library on A33M11x
* @version	1.0
* @date		
* @author	ABOV PTS team
*
* Copyright(C) 2020, ABOV Semiconductor
* All rights reserved.
*
************************************************************************
* ABOV Disclaimer
*
*IMPORTANT NOTICE ? PLEASE READ CAREFULLY
*ABOV Semiconductor ("ABOV") reserves the right to make changes, corrections, enhancements, 
*modifications, and improvements to ABOV products and/or to this document at any time without notice. 
*ABOV does not give warranties as to the accuracy or completeness of the information included herein.
*Purchasers should obtain the latest relevant information of ABOV products before placing orders. 
*Purchasers are entirely responsible for the choice, selection, and use of ABOV products and 
*ABOV assumes no liability for application assistance or the design of purchaser products. No license, 
*express or implied, to any intellectual property rights is granted by ABOV herein. 
*ABOV disclaims all express and implied warranties and shall not be responsible or
*liable for any injuries or damages related to use of ABOV products in such unauthorized applications. 
*ABOV and the ABOV logo are trademarks of ABOV.
*All other product or service names are the property of their respective owners. 
*Information in this document supersedes and replaces the information previously
*supplied in any former versions of this document.
*2020 ABOV Semiconductor  All rights reserved
*
**********************************************************************/

/* Includes ------------------------------------------------------------------- */
#include "A33M11x_hal_scu.h"
#include "A33M11x_hal_mpwm.h"
#include "A33M11x_hal_libcfg.h"



/* Public Functions ----------------------------------------------------------- */
/* MPWM Init/DeInit functions -------------------------------------------------*/
/********************************************************************//**
 * @brief		Initializes the MPWMx peripheral according to the specified
 *               parameters.
 * @param[in]	MPWMx	MPWM peripheral selected, should be:
 * 					- MPWMx	:MPWM0~1 peripheral
 * @return 		HAL Status
 * 					- HAL_OK
 * 					- HAL_ERROR
 *********************************************************************/
HAL_Status_Type HAL_MPWM_Init(MPWM_Type *MPWMx)
{
	/* Check MPWM  handle */
         if(MPWMx == NULL)
        {
            return HAL_ERROR;
         }

	SYST_ACCESS_EN();
	
	if(MPWMx == MPWM0)
	{
		SCU->PER2&=~(1UL<<16);
		SCU->PCER2&=~(1UL<<16);
		
		/* Set up peripheral clock for MPWM0 module */
		SCU->PER2|=(1UL<<16);
		SCU->PCER2|=(1UL<<16);
	}
	else if(MPWMx == MPWM1)
	{
		SCU->PER2&=~(1UL<<17);
		SCU->PCER2&=~(1UL<<17);
		
		/* Set up peripheral clock for MPWM1 module */
		SCU->PER2|=(1UL<<17);
		SCU->PCER2|=(1UL<<17);
	}
	
	SYST_ACCESS_DIS();
	
// PWMEN bit = '1'
	MPWMx->CR1 = 0
		|(0<<8)                    // IRQN[2:0]
		|(1<<0)                    // PWMEN
		;

	MPWMx->CR2 = 0;         				// PSTART disable 
	MPWMx->OLR = 0;         				// MPWM Output Level 
	MPWMx->FOLR = 0;       				// MPWM Force Output Level
	MPWMx->PCR = 0;        				// Protection 0 Control 
	MPWMx->PSR = (0xca<<8) | 0;        	// Protection 0 Status  
	MPWMx->OCR = 0;        				// Protection 1 Control 
	MPWMx->OSR = (0xac<<8) | 0;        	// Protection 1 Status  
	MPWMx->ATR1 = 0;
	MPWMx->ATR2 = 0;
	MPWMx->ATR3 = 0;
	MPWMx->ATR4 = 0;
	MPWMx->ATR5 = 0;
	MPWMx->ATR6 = 0;	
	return HAL_OK;

}


/*********************************************************************//**
 * @brief		De-initializes the MPWMx peripheral registers to their
 *              default reset values.
 * @param[in]	MPWMx	MPWM peripheral selected, should be:
 * 					- MPWMx	:MPWM0~1 peripheral
 * @return 		HAL Status
 * 					- HAL_OK
 * 					- HAL_ERROR
 **********************************************************************/
HAL_Status_Type HAL_MPWM_DeInit(MPWM_Type* MPWMx)
{
	/* Check MPWM  handle */
         if(MPWMx == NULL)
        {
            return HAL_ERROR;
         }

	MPWMx->CR1 = 0; // disable PWMEN
	
	SYST_ACCESS_EN();
	
	if (MPWMx == MPWM0)
	{
		SCU->PER2&=~(1UL<<16);
		SCU->PCER2&=~(1UL<<16);
	}
	else if(MPWMx == MPWM1)
	{
		SCU->PER2&=~(1UL<<17);
		SCU->PCER2&=~(1UL<<17);
	}
	
	SYST_ACCESS_DIS();
	return HAL_OK;
}

/*********************************************************************//**
 * @brief		Enable or Disable PWM start
 * @param[in]	MPWMx	MPWM peripheral selected, should be:
 * 					- MPWMx	:MPWM0~1 peripheral
 * @param[in]	NewState New State of MPWMx peripheral's operation, should be:
 * 					- ENABLE
 * 					- DISABLE
 * @return HAL Status
 * 					- HAL_OK
 * 					- HAL_ERROR
 **********************************************************************/
HAL_Status_Type HAL_MPWM_Start(MPWM_Type* MPWMx, FunctionalState NewState)
{
	/* Check MPWM  handle */
         if(MPWMx == NULL)
        {
            return HAL_ERROR;
         }

	if (NewState == ENABLE) {
		MPWMx->CR2 = (1UL<<0);
	}
	else 	{
		MPWMx->CR2 = (0UL<<0);
	}
	return HAL_OK;
}


/*********************************************************************//**
 * @brief		Enable or Disable PWM Halt
 * @param[in]	MPWMx	MPWM peripheral selected, should be:
 * 					- MPWMx	:MPWM0~1 peripheral
 * @param[in]	NewState New State of MPWMx peripheral's operation, should be:
 * 					- ENABLE
 * 					- DISABLE
 * @return HAL Status
 * 					- HAL_OK
 * 					- HAL_ERROR
 **********************************************************************/
HAL_Status_Type HAL_MPWM_Halt(MPWM_Type* MPWMx, FunctionalState NewState)
{
	uint32_t	tmp;
	
	/* Check MPWM  handle */
	 if(MPWMx == NULL)
	{
			return HAL_ERROR;
	 }

	 tmp = MPWMx->CR2;
	 tmp &= ~(1<<7);
	 
	if (NewState == ENABLE) {
		MPWMx->CR2 |= (tmp | (1UL<<7));
	}
	else 	{
		MPWMx->CR2 |= (tmp | (0UL<<7));
	}
	return HAL_OK;
}



/**********************************************************************//**
 * @brief		Set PWM Output Control data
 * @param[in]	MPWMx	MPWM peripheral selected, should be:
 * 				- MPWMx	:MPWM0~1 peripheral
 * @param[in]	pha_val	
 * 					- 0 = PH_U 
 * 					- 1 = PH_V
 * 					- 2 = PH_W
 * @param[in]	outstatus	PWM output status level control 
 *					- 0 = OUTSTAT_STOP: Selection of the output level when PWMxH/L is set disabled (x = U, V, W)
 *					- 1 = OUTSTAT_START_INVERT : Whether or not to invert the output at the start of PWMxH/L (x = U, V, W)
 * @param[in]	side_val	Protection output at the H or L side 
 *					- 0 = LOW_SIDE: L-side
 *					- 1 = HIGH_SIDE : H-side
 *					- 2 = BOTH_SIDE : L-side & H-side
 * @param[in]	out_val	 output level selection
 * 				When setting the status when pwm is stopped ...
 *					- 0 = LOW_LEVEL : Output  level "L"
 *					- 1 = HIGH_LEVEL : Output  level "H"
  * 				When setting the state to invert the output when pwm starts ..
	*				- 0 = BASIC_OUTPUT : Basic output level(Output  level "L")
 *					- 1 = INVERT_OUTPUT : Inverted output level(Output  level "H")
 * @return 		HAL Status
 * 				- HAL_OK
 * 				- HAL_ERROR
 **********************************************************************/
HAL_Status_Type HAL_MPWM_SetOutput(MPWM_Type* MPWMx, MPWM_PH_VAL pha_val, MPWM_OUTSTAT outstatus, uint8_t side_val, uint8_t out_val)
{
	uint32_t tmp, num, stat;
	
	/* Check MPWM  handle */
	if(MPWMx == NULL)
	{
			return HAL_ERROR;
	}
	
	if (pha_val == PH_U)	num=0;
	else if (pha_val == PH_V) num=1;
	else if (pha_val == PH_W) num=2;
	
	if(outstatus==OUTSTAT_STOP) stat=8;
	else if(outstatus==OUTSTAT_START_INVERT) stat=0;
	
	tmp = MPWMx->OLR;
	
	 if(side_val == LOW_SIDE)
	 {
		 tmp &= ~(1<<(num+stat));	// Phase-x L-side output level initialization (level "L")
		 if(out_val == (HIGH_LEVEL || INVERT_OUTPUT))	
		 {
			 tmp |= (1<<(num+stat));	
		 }
	}
	else if(side_val == HIGH_SIDE)
	{
		tmp &= ~(1<<(num+3+stat));	// Phase-x H-side output level initialization (level "L")
		if(out_val == (HIGH_LEVEL || INVERT_OUTPUT))
		{
			tmp |= (1<<(num+3+stat));
		}
	}
	else if(side_val == BOTH_SIDE) 
	{
		tmp &= ~((1<<(num+stat))|(1<<(num+3+stat)));	// Phase-x L-side & H-side output level initialization (level "L")
		if(out_val == (HIGH_LEVEL || INVERT_OUTPUT))
		{
			tmp |= (1<<(num+stat)) | (1<<(num+3+stat));
		}
	}
	 
	MPWMx->OLR = tmp;
	return HAL_OK;
}

/**********************************************************************//**
 * @brief		Set PWM Force Output Control data
 * @param[in]	MPWMx	MPWM peripheral selected, should be:
 * 				- MPWMx	:MPWM0~1 peripheral
 * @param[in]	pha_val	
 * 					- 0 = PH_U 
 * 					- 1 = PH_V
 * 					- 2 = PH_W
 * @param[in]	side_val	Protection output at the H or L side 
 *					- 0 = LOW_SIDE: L-side
 *					- 1 = HIGH_SIDE : H-side
 *					- 2 = BOTH_SIDE : L-side & H-side
 * @param[in]	fout_val	forced output level selection
 *					- 0 = LOW_LEVEL : Output force level "L"
 *					- 1 = HIGH_LEVEL : Output force level "H"
 * @return 		HAL Status
 * 				- HAL_OK
 * 				- HAL_ERROR
 **********************************************************************/
HAL_Status_Type HAL_MPWM_SetFOutput(MPWM_Type* MPWMx, MPWM_PH_VAL pha_val, uint8_t side_val, uint8_t fout_val)
{
	uint32_t num,tmp;
	
	/* Check MPWM  handle */
	 if(MPWMx == NULL)
	{
			return HAL_ERROR;
	 }
	
	if (pha_val == PH_U)	num=0;
	else if (pha_val == PH_V) num=1;
	else if (pha_val == PH_W) num=2;
	
	tmp = MPWMx->FOLR;
	 
	 if(side_val == LOW_SIDE)
	 {
		 tmp &= ~(1<<num);	// Phase-x L-side output force level initialization (level "L")
		 if(fout_val == HIGH_LEVEL)	
		 {
			 tmp |= (1<<num);	
		 }
	}
	else if(side_val == HIGH_SIDE)
	{
		tmp &= ~(1<<(num+3));	// Phase-x H-side output force level initialization (level "L")
		if(fout_val == HIGH_LEVEL)
		{
			tmp |= (1<<(num+3));
		}
	}
	else if(side_val == BOTH_SIDE) 
	{
		tmp &= ~((1<<num)|(1<<(num+3)));	// Phase-x L-side & H-side output force level initialization (level "L")
		if(fout_val == HIGH_LEVEL)
		{
			tmp |= (1<<num) | (1<<(num+3));
		}
	}
	 
	MPWMx->FOLR =	tmp;
	return HAL_OK;
}


/**********************************************************************//**
 * @brief		Set PWM Protection Control data
 * @param[in]	MPWMx	MPWM peripheral selected, should be:
 * 				- MPWMx	:MPWM0~1 peripheral
 * @param[in]	prot_sel	Protection selection
 *					- 0 = PROTSEL_PROT: protection register
 *					- 1 =PROTSEL_OVV : overvoltage register
 * @param[in]	pha_val	
 * 					- 0 = PH_U 
 * 					- 1 = PH_V
 * 					- 2 = PH_W
 * @param[in]	pol	Protection input polarity selection
 *					- 0 = PROTPOL_LOW_ACTIVE: L-active
 *					- 1 =PROTPOL_HIGH_ACTIVE : H-active
 * @param[in]	dbc_val	Protection input debounce
 *					- 0 ~ 7
 * @param[in]	NewState New state of specified MPWM protection,
 * 				should be:
 * 					- ENABLE	:Enable this MPWM protection.
 * 					- DISABLE	:Disable this MPWM protection.
 * @return 		HAL Status
 * 				- HAL_OK
 * 				- HAL_ERROR
 **********************************************************************/
HAL_Status_Type HAL_MPWM_SetProtCtrl(MPWM_Type* MPWMx, MPWM_PROTSEL prot_sel, MPWM_PH_VAL pha_val, MPWM_PROTPOL pol, uint32_t dbc_val, FunctionalState NewState)
{
	uint32_t	tmp,num;
	
	/* Check MPWM  handle */
	 if(MPWMx == NULL)
	{
			return HAL_ERROR;
	 }
	 
	if (pha_val == PH_U)	num=0;
	else if (pha_val == PH_V) num=1;
	else if (pha_val == PH_W) num=2;
	
	if(prot_sel == PROTSEL_PROT)  tmp = MPWMx->PCR;
	else if(prot_sel == PROTSEL_OVV)  tmp = MPWMx->OCR;	
	tmp &= ~(0xff<<(8*(num+1)));
	tmp |= (pol<<(14+(num*8))) | (NewState<<(15+(num*8))) | (dbc_val<<(8+(num*8)));
	
	if(prot_sel == PROTSEL_PROT) MPWMx->PCR = tmp;
	else if(prot_sel == PROTSEL_OVV)	MPWMx->OCR = tmp;
	 
	return HAL_OK;
}


/**********************************************************************//**
 * @brief		Set PWM Protection Control data
 * @param[in]	MPWMx	MPWM peripheral selected, should be:
 * 				- MPWMx	:MPWM0~1 peripheral
 * @param[in]	prot_sel	Protection selection
 *					- 0 = PROTSEL_PROT: protection register
 *					- 1 =PROTSEL_OVV : overvoltage register
 * @param[in]	pha_val	
 * 					- 0 = PH_U 
 * 					- 1 = PH_V
 * 					- 2 = PH_W
 * @param[in]	side_val	Protection output at the H or L side 
 *					- 0 = LOW_SIDE: L-side
 *					- 1 =HIGH_SIDE : H-side
 *					- 2 =BOTH_SIDE : L-side & H-side
 * @param[in]	NewState New state of specified MPWM protection,
 * 				should be:
 * 					- ENABLE	:Enable this MPWM protection output.
 * 					- DISABLE	:Disable this MPWM protection output.
 * @return 		HAL Status
 * 				- HAL_OK
 * 				- HAL_ERROR
 **********************************************************************/
HAL_Status_Type HAL_MPWM_SetProtOutput(MPWM_Type* MPWMx, MPWM_PROTSEL prot_sel, MPWM_PH_VAL pha_val, uint8_t side_val, FunctionalState NewState)
{
	uint32_t	tmp,num;
	
	/* Check MPWM  handle */
	 if(MPWMx == NULL)
	{
			return HAL_ERROR;
	 }
	 
	if (pha_val == PH_U)	num=0;
	else if (pha_val == PH_V) num=1;
	else if (pha_val == PH_W) num=2;
	
	if (prot_sel == PROTSEL_PROT) tmp = MPWMx->PCR;
	else if(prot_sel == PROTSEL_OVV) tmp = MPWMx->OCR;
	 
	if(side_val == LOW_SIDE)	
	{
		tmp &= ~(1<<num);	// Phase-x L-side initialization
		if(NewState == ENABLE) tmp |= (1<<num);
	}
	else if(side_val == HIGH_SIDE) 
	{
		tmp &= ~(1<<(num+3));	// Phase-x H-side initialization
		if(NewState == ENABLE) tmp |= (1<<(num+3));
	}
	else if(side_val ==	BOTH_SIDE) 
	{
		tmp &= ~((1<<num)|(1<<(num+3)));	// Phase-x L-side & H-side initialization
		if(NewState == ENABLE) tmp |= ((1<<num)|(1<<(num+3)));
	}
	
	if(prot_sel == PROTSEL_PROT) MPWMx->PCR = tmp;
	else if(prot_sel == PROTSEL_OVV) MPWMx->OCR = tmp;
	
	return HAL_OK;
}

/**********************************************************************//**
 * @brief		Clear PWM Protection Status data
 * @param[in]	MPWMx	MPWM peripheral selected, should be:
 * 					- MPWMx	:MPWM0~1 peripheral
 * @param[in]	prot_sel	Protection selection
 *					- 0 = PROTSEL_PROT: protection register
 *					- 1 =PROTSEL_OVV : overvoltage register
 * @param[in]	Data	Data to control
 *						Protection Register
 *						- MPWM_PSR_PROTIF :  Protection interrupt status
 *						- MPWM_PSR_WHPROT :  Phase W's H-side protection flag
 *						- MPWM_PSR_VHPROT :  Phase V's H-side protection flag
 *						- MPWM_PSR_UHPROT :  Phase U's H-side protection flag
 *						- MPWM_PSR_WLPROT :  Phase W's L-side protection flag
 *						- MPWM_PSR_VLPROT :  Phase V's L-side protection flag
 *						- MPWM_PSR_ULPROT :  Phase U's L-side protection flag
  *						Overvoltage Register
 *						- MPWM_OSR_OVINIF :  Overvoltage interrupt status
 *						- MPWM_OSR_WHOVINF :  Phase W's H-side overvoltage flag
 *						- MPWM_OSR_VHOVINF :  Phase V's H-side overvoltage flag
 *						- MPWM_OSR_UHOVINF :  Phase U's H-side overvoltage flag
 *						- MPWM_OSR_WLOVINF :  Phase W's L-side overvoltage flag
 *						- MPWM_OSR_VLOVINF :  Phase V's L-side overvoltage flag
 *						- MPWM_OSR_ULOVINF :  Phase U's L-side overvoltage flag
 * @return 		HAL Status
 * 					- HAL_OK
 * 					- HAL_ERROR
 **********************************************************************/
HAL_Status_Type HAL_MPWM_ProtStatClear(MPWM_Type* MPWMx, MPWM_PROTSEL prot_sel, uint32_t Data)
{
	/* Check MPWM  handle */
	if(MPWMx == NULL)
	{
		return HAL_ERROR;
	}

	if (prot_sel == PROTSEL_PROT){
		MPWMx->PSR = (0xca<<8) | (Data&0xff);        // Protection 0 Status  		
	}
	else if (prot_sel == PROTSEL_OVV){	
		MPWMx->OSR = (0xac<<8) | (Data&0xff);        // Protection 1 Status  			
	}
	return HAL_OK;
}


/**********************************************************************//**
 * @brief		Get PWM Counter data
 * @param[in]	MPWMx	MPWM peripheral selected, should be:
 * 					- MPWMx	:MPWM0~1 peripheral
 * @param[in]	None
 * @return 		Counter Value
 **********************************************************************/
uint32_t HAL_MPWM_GetCounter(MPWM_Type *MPWMx)
{
	return (MPWMx->CNT);
}


/********************************************************************//**
 * @brief 		Enable or disable specified MPWM interrupt.
 * @param[in]	MPWMx	MPWM peripheral selected, should be:
 * 					- MPWMx	:MPWM0~1 peripheral
 * @param[in]	MPIntCfg	Specifies the interrupt src
*						- MPWM_IER_PRDWIE : Phase-W period interrupt enable
*						- MPWM_IER_BOTWIE :Phase-W bottom interrupt enable
*						- MPWM_IER_PRDVIE : Phase-V period interrupt enable
*						- MPWM_IER_BOTVIE : Phase-V bottom interrupt enable
*						- MPWM_IER_PRDUIE : Phase-U period interrupt enable 
*						- MPWM_IER_BOTUIE :Phase-U bottom interrupt enable 
*						- MPWM_IER_WHIE : WH duty or ATR6 match Interrupt enable
*						- MPWM_IER_VHIE	 : VH duty or ATR5 match Interrupt enable
*						- MPWM_IER_UHIE	 : UH duty or ATR4 match Interrupt enable
*						- MPWM_IER_WLIE : WL duty or ATR3 match Interrupt enable
*						- MPWM_IER_VLIE  :VL duty or ATR2match Interrupt enable 
*						- MPWM_IER_ULIE	 :UL duty or ATR1 match Interrupt enable 
 * @param[in]	NewState New state of specified MPWM interrupt type,
 * 				should be:
 * 					- ENABLE	:Enable this MPWM interrupt type.
 * 					- DISABLE	:Disable this MPWM interrupt type.
 * @return 		HAL Status
 * 					- HAL_OK
 * 					- HAL_ERROR
 *********************************************************************/
HAL_Status_Type HAL_MPWM_IntConfig(MPWM_Type* MPWMx, uint32_t MPIntCfg, FunctionalState NewState)
{
	/* Check MPWM  handle */
         if(MPWMx == NULL)
        {
            return HAL_ERROR;
         }

	if (NewState == ENABLE)
	{
		MPWMx->IER |= MPIntCfg;
	}
	else
	{
		MPWMx->IER &= (~MPIntCfg) & 0xfff;
	}
	return HAL_OK;
}


/********************************************************************//**
 * @brief 		MPWM interrupt status clear
 * @param[in]	MPWMx	MPWM peripheral selected, should be:
 * 					- MPWMx	:MPWM0~1 peripheral
 * @param[in]	Type	Specifies the interrupt src
*						- MPWM_SR_PRDWIF : Phase W's PWM period interrupt flag
*						- MPWM_SR_BOTWIF : Phase W's PWM bottom interrupt flag
*						- MPWM_SR_PRDVIF : Phase V's PWM period interrupt flag
*						- MPWM_SR_BOTVIF : Phase V's PWM bottom interrupt flag
*						- MPWM_SR_PRDUIF : Phase U's PWM period interrupt flag 
*						- MPWM_SR_BOTUIF : Phase U's PWM bottom interrupt flag
*						- MPWM_SR_DWHIF_ATR6F : PWM duty WH interrupt flag
*						- MPWM_SR_DVHIF_ATF5F	 : PWM duty VH interrupt flag
*						- MPWM_SR_DUHIF_ATR4F	 : PWM duty UH interrupt flag
*						- MPWM_SR_DWLIF_ATF3F : PWM duty WL interrupt flag
*						- MPWM_SR_DVLIF_ATR2F  :PWM duty VL interrupt flag 
*						- MPWM_SR_DULIF_ATF1F	 : PWM duty UL interrupt flag
 * @param[in]	NewState New state of specified MPWM interrupt type,
 * 				should be:
 * 					- ENABLE	:Enable this MPWM interrupt type.
 * 					- DISABLE	:Disable this MPWM interrupt type.
 * @return 		HAL Status
 * 					- HAL_OK
 * 					- HAL_ERROR
 *********************************************************************/
HAL_Status_Type HAL_MPWM_IntStatClear(MPWM_Type* MPWMx, uint32_t Type)
{
	/* Check MPWM  handle */
	 if(MPWMx == NULL)
	{
			return HAL_ERROR;
	 }

	MPWMx->SR |= Type;
	 
	return HAL_OK;
}

/********************************************************************//**
 * @brief 		MPWM interrupt status get
 * @param[in]	MPWMx	MPWM peripheral selected, should be:
 * 					- MPWMx	:MPWM0~1 peripheral
 * @return 		SR register value 
 *********************************************************************/
uint32_t HAL_MPWM_GetIntStat(MPWM_Type* MPWMx)
{
	
	/* Check MPWM  handle */
	 if(MPWMx == NULL)
	{
			return HAL_ERROR;
	 }
	 
	return MPWMx->SR;
}


/********************************************************************//**
 * @brief 		Enable or disable specified MPWM Protection interrupt.
 * @param[in]	MPWMx	MPWM peripheral selected, should be:
 * 					- MPWMx	:MPWM0~1 peripheral
 * @param[in]	ProtIntCfg	Protection the interrupt src
*						- 0 = PROTSEL_PROT : Protection interrupt enable
*						- 1 = PROTSEL_OVV : Overvoltage interrupt enable
 * @param[in]	NewState New state of specified MPWM interrupt type,
 * 				should be:
 * 					- ENABLE	:Enable this MPWM interrupt type.
 * 					- DISABLE	:Disable this MPWM interrupt type.
 * @return 		HAL Status
 * 					- HAL_OK
 * 					- HAL_ERROR
 *********************************************************************/
HAL_Status_Type HAL_MPWM_ProtIntConfig(MPWM_Type* MPWMx, MPWM_PROTSEL ProtIntCfg, FunctionalState NewState)
{
	/* Check MPWM  handle */
	 if(MPWMx == NULL)
	{
			return HAL_ERROR;
	 }

	if(ProtIntCfg == PROTSEL_PROT)
	{
		MPWMx->PCR &= ~(1<<7);
		
		if(NewState == ENABLE)	MPWMx->PCR |= (1<<7);
	}
	else if(ProtIntCfg == PROTSEL_OVV)
	{
		MPWMx->OCR &= ~(1<<7);
		
		if(NewState == ENABLE) MPWMx->OCR |= (1<<7);
	}
	return HAL_OK;
}


/********************************************************************//**
 * @brief 		MPWM interrupt status get
 * @param[in]	MPWMx	MPWM peripheral selected, should be:
 * 					- MPWMx	:MPWM0~1 peripheral
 * @param[in]	ProtIntCfg	Protection the interrupt src
*						- 0 = PROTSEL_PROT : Protection interrupt enable
*						- 1 = PROTSEL_OVV : Overvoltage interrupt enable
 * @return 		PCR or OCR register value 
 *********************************************************************/
uint32_t HAL_MPWM_GetProtIntStat(MPWM_Type* MPWMx,MPWM_PROTSEL ProtIntCfg)
{
	
	/* Check MPWM  handle */
	 if(MPWMx == NULL)
	{
			return HAL_ERROR;
	 }
	
	if(ProtIntCfg == PROTSEL_PROT)	return MPWMx->PCR;
	else return MPWMx->OCR;
	 
}

/********************************************************************//**
 * @brief 		Configure PWM Mode setting.
 * @param[in]	MPWMx	MPWM peripheral selected, should be:
 * 					- MPWMx	:MPWM0~1 peripheral
 * @param[in]	motorb			MPWM mode selection
 *						- 0 = MOTOR_PWM
 *						- 1 = NORMAL_PWM
 *						- 2 = INDIVIDUAL_PWM
 * @param[in]	update			0:duty and period after match, 1:after 2 PWM clocks
 *						- 0 = PRD_MAT : Updates are performed at designated timings.
 *						- 1 = PWM_CLOCK : The duty and period are updated immediately when requested.
 *										With this setting, the duty and period registers are updated after two PWM clock cycles.						
 * @param[in]	tup				Whether to enable or disable period/duty updates (at period matches)
 *						- 0 = UPDATE_DISABLE :Periods and duties are not updated at every period match.
 *						- 1 = UPDATE_PERIOD : Periods and duties are updated at every period match.
 * @param[in]	bup				Whether to enable or disable period/duty updates (at bottom matches)
 *						- 0 = UPDATE_DISABLE :Periods and duties are not updated at every bottom match.
 *						- 1 = UPDATE_BOTTOM : Periods and duties are updated at every bottom match.
 * @param[in]	mchmode	Channel symmetry/asymmetry mode selection
 *						- 0 = SYMMETRIC_2CH 
 *						- 1 = ASYMMETRIC_1CH 
 *						- 2 = SYMMETRIC_1CH 
 * @param[in]	updown		PWM up/down counter mode selection
 *						- 0 = UP_COUNT 
 *						- 1 = UP_DOWN_COUNT  
 * @return 		HAL Status
 * 					- HAL_OK
 * 					- HAL_ERROR
 *********************************************************************/
HAL_Status_Type HAL_MPWM_Cmd(MPWM_Type* MPWMx,uint32_t motorb, uint32_t update, uint32_t tup, uint32_t bup, uint32_t mchmode, uint32_t updown)
{
	/* Check MPWM  handle */
	 if(MPWMx == NULL)
	{
			return HAL_ERROR;
	 }

	MPWMx->MR = 0
	|((motorb&0x03)<<14)				// MOTORB    0:motor mode, 1:normal mode, 3:Extend mode	
	|((update&0x01)<<7)					// UPDATE    0:duty and period after match, 1:after 2 PWM clocks
	|((tup&0x01)<<5)					// TUP        0:every period match, 1:every period match		
	|((bup&0x01)<<4)					// BUP        0:every period match, 1:every bottom match	
	|((mchmode&0x03)<<1)				// MCHMOD  0:2-ch symmetric, 1:1-ch asymmetric, 2:1-ch symmetric
	|((updown)<<0)						// UPDOWN  0: PWM UP count, 1:PWM UP/DOWN 
	;
	return HAL_OK;
}


/********************************************************************//**
 * @brief 		Set period data .
 * @param[in]	MPWMx	MPWM peripheral selected, should be:
 * 					- MPWMx	:MPWM0~1 peripheral
 * @param[in]	period			MPWM period data									
 * @return 		HAL Status
 * 					- HAL_OK
 * 					- HAL_ERROR
 *********************************************************************/
HAL_Status_Type HAL_MPWM_SetPeriod(MPWM_Type* MPWMx, uint32_t period)
{
	/* Check MPWM  handle */
         if(MPWMx == NULL)
        {
            return HAL_ERROR;
         }

	MPWMx->PRD = (period&0xffff);  // period. it sould be larger than 0x10		
	return HAL_OK;
}

/********************************************************************//**
 * @brief 		Set duty U high/low data .
 * @param[in]	MPWMx	MPWM peripheral selected, should be:
 * 					- MPWMx	:MPWM0~1 peripheral
 * @param[in]	udutyH			MPWM UdutyH data
 * @param[in]	udutyL			MPWM UdutyL data			 							
 * @return 		HAL Status
 * 					- HAL_OK
 * 					- HAL_ERROR
 *********************************************************************/
HAL_Status_Type HAL_MPWM_SetUDuty(MPWM_Type* MPWMx, uint32_t udutyH, uint32_t udutyL)
{
	/* Check MPWM  handle */
         if(MPWMx == NULL)
        {
            return HAL_ERROR;
         }

	MPWMx->DUH = ((udutyH) &0xffff);  // if using I/O control function, set period data	
	MPWMx->DUL = ((udutyL) &0xffff); 	
	return HAL_OK;
}

/********************************************************************//**
 * @brief 		Set duty V high/low data .
 * @param[in]	MPWMx	MPWM peripheral selected, should be:
 * 					- MPWMx	:MPWM0~1 peripheral
 * @param[in]	vdutyH			MPWM VdutyH data
 * @param[in]	vdutyL			MPWM VdutyL data			 							
 * @return 		HAL Status
 * 					- HAL_OK
 * 					- HAL_ERROR
 *********************************************************************/
HAL_Status_Type HAL_MPWM_SetVDuty(MPWM_Type* MPWMx, uint32_t vdutyH, uint32_t vdutyL)
{
	/* Check MPWM  handle */
         if(MPWMx == NULL)
        {
            return HAL_ERROR;
         }

	MPWMx->DVH = ((vdutyH) &0xffff);  // if using I/O control function, set period data	
	MPWMx->DVL = ((vdutyL) &0xffff); 	
	return HAL_OK;
}

/********************************************************************//**
 * @brief 		Set duty W high/low data .
 * @param[in]	MPWMx	MPWM peripheral selected, should be:
 * 					- MPWMx	:MPWM0~1 peripheral
 * @param[in]	wdutyH			MPWM WdutyH data
 * @param[in]	wdutyL			MPWM WdutyL data			 							
 * @return 		HAL Status
 * 					- HAL_OK
 * 					- HAL_ERROR
 *********************************************************************/
HAL_Status_Type HAL_MPWM_SetWDuty(MPWM_Type* MPWMx, uint32_t wdutyH, uint32_t wdutyL)
{
	/* Check MPWM  handle */
         if(MPWMx == NULL)
        {
            return HAL_ERROR;
         }

	MPWMx->DWH = ((wdutyH) &0xffff);  // if using I/O control function, set period data	
	MPWMx->DWL = ((wdutyL) &0xffff); 
	return HAL_OK;
}

/********************************************************************//**
 * @brief 		Set dead time
 * @param[in]	MPWMx	MPWM peripheral selected, should be:
 * 					- MPWMx	:MPWM0~1 peripheral
 * @param[in]	dten			dead time enable 
 * 					- 0 = DTIME_DIS : Disables the dead time function
 * 					- 1 = DTIME_EN : Enables the dead time function
 * @param[in]	pshrt		Protect short condition 
 * 					- 0 = PSHRT_EN : Enables short-circuit protection
 *									 (When the H- and L-side outputs are active simultaneously, they are all turned off.)
 * 					- 1 = PSHRT_DIS : Disables short-circuit protection
 * @param[in]	clk			Deadtime prescaler
 *						- 0 = PWMCLK_2 :PWM CLK /2
 *						- 1 = PWMCLK_4 :PWM CLK /4
 *						- 2 = PWMCLK_8 :PWM CLK /8
 *						- 3 = PWMCLK_16 :PWM CLK /16
 * @param[in]	clkdata	Rising or falling edge dead time value (set as the delay time from the normal polarity to the falling edge level output)	
 *							- 0x01~0xff dead time value
 * @param[in]	dirsel		Dead time mode selection	
 * 						- 0 = LEAD_EDGE_PWMxH : Inserts dead time at the leading edge of PWMxH and the trailing edge of PWMxL
 * 						- 1 = TRAILL_EDGE_PWMxH : Inserts dead time at the trailing edge of PWMxH and the leading edge of PWMxL
 * @return 		HAL Status
 * 					- HAL_OK
 * 					- HAL_ERROR
 *********************************************************************/
HAL_Status_Type HAL_MPWM_SetDeadTime(MPWM_Type* MPWMx, uint32_t dten, uint32_t pshrt, uint32_t clk, uint32_t clkdata, uint32_t dirsel)
{
	/* Check MPWM  handle */
         if(MPWMx == NULL)
        {
            return HAL_ERROR;
         }

 	MPWMx->DTR = 0
	|((dten&0x01)<<15)                 	// DTEN
	|((pshrt&0x01)<<14)                	// PSHRT	
	|((dirsel&0x01)<<13)				// DTMDSEL
	|((clk&0x03)<<8)                   	// DTCLK
	|((clkdata&0xff)<<0)               	// DT
	;
	return HAL_OK;
}

/**********************************************************************//**
 * @brief		Set PWM ADC Trigger Counter
 * @param[in]	MPWMx	MPWM peripheral selected, should be:
 * 					- MPWMx	:MPWM0~1 peripheral
 * @param[in]	trgsrc	ADC trigger source counter		
 * 					- 0 = PH_U 
 * 					- 1 = PH_V
 * 					- 2 = PH_W
 * @param[in]	ch	channel(0~6)
 * @param[in]	atudt	Trigger register update mode
 *						- 0=TRG_PRD_MAT : ADC trigger value applied at period match event 
 *						- 1=TRG_PWM_CLOCK : Trigger register update mode
 * @param[in]	atmod	ADC trigger Mode register
 *						- 0=TRG_DISABLE : ADC trigger Disable
 *						- 1=TRG_UP_COUNT : Trigger out when up count match
 *						- 2=TRG_DOWN_COUNT : Trigger out when down count match
 *						- 3=TRG_UP_DOWN_COUNT : Trigger out when up-down count match
 * @param[in]	atcnt	ADC Trigger counter
 * @return 		HAL Status
 * 					- HAL_OK
 * 					- HAL_ERROR
 **********************************************************************/
HAL_Status_Type HAL_MPWM_SetATR(MPWM_Type* MPWMx, MPWM_PH_VAL trgsrc,uint32_t ch, MPWM_ADCTRG_TIME atudt, MPWM_ADCTRG_MODE atmod, uint32_t atcnt)
{
	uint32_t tempreg;
	
	/* Check MPWM  handle */
         if(MPWMx == NULL)
        {
            return HAL_ERROR;
         }
	
	tempreg = 0
	|((trgsrc&0x03)<<22)
	|((atudt&0x01)<<19)
	|((atmod&0x03)<<16)
	|((atcnt&0xFFFF)<<0)	
	;
	
	if (ch==1){
		MPWMx->ATR1 = tempreg; 
	}
	else if (ch==2){	
		MPWMx->ATR2 = tempreg; 
	}
	else if (ch==3){	
		MPWMx->ATR3 = tempreg; 
	} 
	else if (ch==4){	
		MPWMx->ATR4 = tempreg; 
	} 
	else if (ch==5){	
		MPWMx->ATR5 = tempreg; 
	} 
	else if (ch==6){	
		MPWMx->ATR6 = tempreg; 
	} 
	return HAL_OK;
}




//==================================================================
//
//		Extend Mode Register
//
//==================================================================

/********************************************************************//**
 * @brief 		Phase-X Counter Start/Stop.
 * @param[in]	MPWMx	MPWM peripheral selected, should be:
 * 					- MPWMx	:MPWM0~1 peripheral
 * @param[in]	pha_val			
 * 					- 0 = PH_U 
 * 					- 1 = PH_V
 * 					- 2 = PH_W
 * @param[in]	OperSel			U(0), V(1), W(2)
 *						- 0 = MPWM_Ext_STOP
 *						- 1 = MPWM_Ext_START
 * @return 		HAL Status
 * 					- HAL_OK
 * 					- HAL_ERROR
 *********************************************************************/
HAL_Status_Type HAL_MPWM_ExtStartCmd(MPWM_Type *MPWMx, MPWM_PH_VAL pha_val, MPWM_Ext_OPERATEVAL OperSel)
{
	uint32_t		reg_val, reg_val2;

	/* Check MPWM  handle */
	 if(MPWMx == NULL)
	{
			return HAL_ERROR;
	 }
	reg_val = MPWMx->CR3;
	reg_val2 = 0;
	
	if (pha_val == PH_U)
	{
		reg_val &= ~(1<<1);
		
		if (OperSel == MPWM_Ext_STOP)
		{
			reg_val2 = (1<<1);	// Phase-U Stop
		}
		else if (OperSel == MPWM_Ext_START)
		{
			reg_val |= (0<<1) | (1<<1);	// Phase-U Enable & Start
		}
	}
	else if (pha_val == PH_V)
	{
		reg_val &= ~(1<<9);
		
		if (OperSel == MPWM_Ext_STOP)
		{
			reg_val2 = (1<<9);	// Phase-V Stop
		}
		else if (OperSel == MPWM_Ext_START)
		{
			reg_val |= (0<<8) | (1<<9);	// Phase-W Enable & Start
		}
	}
	else if (pha_val == PH_W)
	{
		reg_val &= ~(1<<17);
		
		if (OperSel == MPWM_Ext_STOP)
		{
			reg_val2 = (1<<17);	// Phase-W stop
		}
		else if (OperSel == MPWM_Ext_START)
		{
			reg_val |= (1<<16) | (1<<17);
		}
	}
	if(OperSel == MPWM_Ext_START)	MPWMx->CR3 = reg_val;
	else if(OperSel == MPWM_Ext_STOP) MPWMx->CR4 = reg_val2;
	return HAL_OK;
}


/********************************************************************//**
 * @brief 		Phase-X Module Enable & IRQ Period Setting.
 * @param[in]	MPWMx	MPWM peripheral selected, should be:
 * 					- MPWMx	:MPWM0~1 peripheral
 * @param[in]	pha_val			phase selection
 * 					- 0 = PH_U 
 * 					- 1 = PH_V
 * 					- 2 = PH_W
 * @param[in]	irq_n			Phase x(u,v,w) interrupt interval setting
 *						- 0x0 ~ 0x7
 * @param[in]	Phase-X			U(0), V(1), W(2)
 * @return 		HAL Status
 * 					- HAL_OK
 * 					- HAL_ERROR
 *********************************************************************/
HAL_Status_Type HAL_MPWM_ExtEnableCmd(MPWM_Type *MPWMx, MPWM_PH_VAL pha_val, uint32_t irq_n, FunctionalState NewStatus)
{
	uint32_t		reg_val, reg_val2;

	/* Check MPWM  handle */
	 if(MPWMx == NULL)
	{
			return HAL_ERROR;
	 }

	reg_val = MPWMx->CR3;
	reg_val2 = MPWMx->CR4;
	
	if (pha_val == PH_U)
	{
		reg_val &= ~(7<<4);
		
		if (NewStatus == ENABLE)
		{
			reg_val |= ((1<<0) | ((irq_n & 0x07)<<4));
		}
		else if (NewStatus == DISABLE)
		{
			reg_val2 = (1<<0);
		}
	}
	else if (pha_val == PH_V)
	{

		reg_val &= ~(7<<12);
		
		if (NewStatus == ENABLE)
		{
			reg_val |= ((1<<0) | (1<<8) | ((irq_n & 0x07)<<12));
		}
		else if (NewStatus == DISABLE)
		{
			reg_val2 = (1<<8);
		}
	}
	else if (pha_val == PH_W)
	{
		reg_val &= ~(7<<20);
		
		if (NewStatus == ENABLE)
		{
			reg_val |= ((1<<0) | (1<<16) | ((irq_n & 0x07)<<20));
		}
		else if (NewStatus == DISABLE)
		{
			reg_val2 = (1<<16);
		}
	}
	if (NewStatus == ENABLE) MPWMx->CR3 = reg_val;
	else if(NewStatus == DISABLE) MPWMx->CR4 = reg_val2;

	return HAL_OK;
}



/********************************************************************//**
 * @brief 		Phase-X Halt & Continuous Setting.
 * @param[in]	MPWMx	MPWM peripheral selected, should be:
 * 					- MPWMx	:MPWM0~1 peripheral
 * @param[in]	pha_val			phase selection
 * 					- 0 = PH_U 
 * 					- 1 = PH_V
 * 					- 2 = PH_W
 * @param[in]	PauSel			Phase x(u,v,w) interrupt interval setting
 * 					- 0 = MPWM_Ext_HALT
 *						- 1 = MPWM_Ext_CONT
 * @return 		HAL Status
 * 					- HAL_OK
 * 					- HAL_ERROR
 *********************************************************************/
HAL_Status_Type HAL_MPWM_ExtPauseCmd(MPWM_Type *MPWMx, MPWM_PH_VAL pha_val, MPWM_Ext_PAUSE PauSel)
{
	uint32_t		reg_val, reg_val2;

	/* Check MPWM  handle */
         if(MPWMx == NULL)
        {
            return HAL_ERROR;
         }

	reg_val = MPWMx->CR3;
	reg_val2 = 0;
	
	if (pha_val == PH_U)
	{
		reg_val &= ~(1<<3);
		
		if (PauSel == MPWM_Ext_CONT)
		{
			reg_val2 = (1<<3);
		}
		else if (PauSel == MPWM_Ext_HALT)
		{
			reg_val |= (1<<0) | (1<<3);
		}
	}
	else if (pha_val == PH_V)
	{
		reg_val &= ~(1<<11);
		
		if (PauSel == MPWM_Ext_CONT)
		{
			reg_val2 = (1<<11);
		}
		else if (PauSel == MPWM_Ext_HALT)
		{
			reg_val |= (1<<8) | (1<<11);
		}
	}
	else if (pha_val == PH_W)
	{
		reg_val &= ~(1<<19);
		
		if (PauSel == MPWM_Ext_CONT)
		{
			reg_val2 = (1<<19);
		}
		else if (PauSel == MPWM_Ext_HALT)
		{
			reg_val |= (1<<16) | (1<<19);
		}
	}
	
	if(PauSel==MPWM_Ext_HALT) MPWMx->CR3 = reg_val;
	else if(PauSel==MPWM_Ext_CONT) MPWMx->CR4 = reg_val2;
	
	return HAL_OK;
}


/********************************************************************//**
 * @brief 		Phase-X Priod Setting.
 * @param[in]	MPWMx	MPWM peripheral selected, should be:
 * 					- MPWMx	:MPWM0~1 peripheral
 * @param[in]	pha_val			phase selection
 * 					- 0 = PH_U 
 * 					- 1 = PH_V
 * 					- 2 = PH_W
 * @param[in]	period			 16-bit PWM period
 *						- The bit value must be larger than 0x0010
 * @return 		HAL Status
 * 					- HAL_OK
 * 					- HAL_ERROR
 *********************************************************************/
HAL_Status_Type HAL_MPWM_ExtSetPeriod(MPWM_Type* MPWMx, MPWM_PH_VAL pha_val, uint32_t period)
{
	/* Check MPWM  handle */
         if(MPWMx == NULL)
        {
            return HAL_ERROR;
         }

	if (pha_val == PH_U)
	{
		MPWMx->PRDU = (period & 0xFFFF);
	}
	else if (pha_val == PH_V)
	{
		MPWMx->PRDV = (period & 0xFFFF);
	}
	else if (pha_val == PH_W)
	{
		MPWMx->PRDW = (period & 0xFFFF);
	}
	return HAL_OK;
}


/********************************************************************//**
 * @brief 		Phase-X Get Counter.
 * @param[in]	MPWMx	MPWM peripheral selected, should be:
 * 					- MPWMx	:MPWM0~1 peripheral
 * @param[in]	pha_val			phase selection
 * 					- 0 = PH_U 
 * 					- 1 = PH_V
 * 					- 2 = PH_W
 * @return 		Counter Value
 *********************************************************************/
uint32_t HAL_MPWM_ExtGetCounter(MPWM_Type *MPWMx, MPWM_PH_VAL pha_val)
{
	/* Check MPWM  handle */
         if(MPWMx == NULL)
        {
            return HAL_ERROR;
         }

	if (pha_val == PH_U)
	{
		return (MPWMx->CNTU);
	}
	else if (pha_val == PH_V)
	{
		return (MPWMx->CNTV);
	}
	else if (pha_val == PH_W)
	{
		return (MPWMx->CNTW);
	}
	
	return 0;
}


/********************************************************************//**
 * @brief 		Set dead time
 * @param[in]	MPWMx	MPWM peripheral selected, should be:
 * 					- MPWMx	:MPWM0~1 peripheral
 * @param[in]	pha_val			phase selection
 * 					- 0 = PH_U 
 * 					- 1 = PH_V
 * 					- 2 = PH_W
 * @param[in]	dten			dead time enable 
 * 					- 0 = DTIME_DIS : Disables the dead time function
 * 					- 1 = DTIME_EN : Enables the dead time function
 * @param[in]	pshrt		Protect short condition 
 * 					- 0 = PSHRT_EN : Enables short-circuit protection
 *									 (When the H- and L-side outputs are active simultaneously, they are all turned off.)
 * 					- 1 = PSHRT_DIS : Disables short-circuit protection
 * @param[in]	clk			Deadtime prescaler
 *						- 0 = PWMCLK_2 :PWM CLK /2
 *						- 1 = PWMCLK_4 :PWM CLK /4
 *						- 2 = PWMCLK_8 :PWM CLK /8
 *						- 3 = PWMCLK_16 :PWM CLK /16
  * @param[in]	rclkdata	Rising edge dead time value (set as the delay time from the normal polarity to the rising edge level output)	
 *							- 0x01~0xff dead time value
 * @param[in]	fclkdata	 Falling edge dead time value (set as the delay time from the normal polarity to the falling edge level output)	
 *							- 0x01~0xff dead time value
 * @param[in]	dirsel		Dead time mode selection	
 * 						- 0 = LEAD_EDGE_PWMxH : Inserts dead time at the leading edge of PWMxH and the trailing edge of PWMxL
 * 						- 1 = TRAILL_EDGE_PWMxH : Inserts dead time at the trailing edge of PWMxH and the leading edge of PWMxL							
 * @return 		HAL Status
 * 					- HAL_OK
 * 					- HAL_ERROR
 *********************************************************************/
HAL_Status_Type HAL_MPWM_ExtSetDeadTime(MPWM_Type* MPWMx, MPWM_PH_VAL pha_val, uint32_t dten, uint32_t pshrt, uint32_t clk, uint32_t rclkdata, uint32_t fclkdata, uint32_t dirsel)
{
	uint32_t		reg_val;
	
	/* Check MPWM  handle */
	 if(MPWMx == NULL)
	{
			return HAL_ERROR;
	 }

	reg_val = 0
	|((dirsel&0x01)<<31)			// DTMDSEL
	|((dten&0x01)<<23)				// DTEN
	|((pshrt&0x01)<<22)				// PSHRT
	|((clk&0x03)<<16)				// DTCLK
	|((rclkdata&0xff)<<8)			// RDT
	|((fclkdata&0xff)<<0)			// FDT
	;
	
	if (pha_val == PH_U)
	{
		MPWMx->DTRU = reg_val;
	}
	else if (pha_val == PH_V)
	{
		MPWMx->DTRV = reg_val;
	}
	else if (pha_val == PH_W)
	{
		MPWMx->DTRW = reg_val;
	}
	return HAL_OK;
	
}


/********************************************************************//**
 * @brief 		Capture Counter Enable
 * @param[in]	MPWMx	MPWM peripheral selected, should be:
 * 					- MPWMx	:MPWM0~1 peripheral
 * @param[in]	pha_val			phase selection
 * 					- 0 = PH_U 
 * 					- 1 = PH_V
 * 					- 2 = PH_W
 * @param[in]	ScapEdge Phase Value
 * 					- 0 = SUBCAP_FALLING
 * 					- 1 = SUBCAP_RISING
 * @param[in]	NewStatus Phase Value
 * 					- 0 = DISABLE
 * 					- 1 = ENABLE
 * @return 		HAL Status
 * 					- HAL_OK
 * 					- HAL_ERROR
 *********************************************************************/
HAL_Status_Type HAL_MPWM_ExtCaptureCmd(MPWM_Type *MPWMx, MPWM_PH_VAL pha_val, MPWM_SCAP_EDGE ScapEdge, FunctionalState NewStatus)
{
	uint32_t		reg_val, reg_val2;
	
	/* Check MPWM  handle */
	 if(MPWMx == NULL)
	{
			return HAL_ERROR;
	 }

	
	if (pha_val == PH_U)
	{
		reg_val = MPWMx->CAPCNTU;
		reg_val2 = MPWMx->SCAPU;
		
		reg_val &= ~(1<<27);
		reg_val2 &= ~(1<<28);
		
		if (NewStatus == ENABLE)
		{
			reg_val |= (1<<27);
			
			if (ScapEdge == SUBCAP_RISING)
			{
				reg_val2 |= (0<<28);
			}
			else if (ScapEdge == SUBCAP_FALLING)
			{
				reg_val2 |= (1<<28);
			}
		}
		
		MPWMx->CAPCNTU = reg_val;
		MPWMx->SCAPU = reg_val2;
	}
	else if (pha_val == PH_V)
	{
		reg_val = MPWMx->CAPCNTV;
		reg_val2 = MPWMx->SCAPV;
		
		reg_val &= ~(1<<27);
		reg_val2 &= ~(1<<28);
		
		if (NewStatus == ENABLE)
		{
			reg_val |= (1<<27);
			
			if (ScapEdge == SUBCAP_RISING)
			{
				reg_val2 |= (0<<28);
			}
			else if (ScapEdge == SUBCAP_FALLING)
			{
				reg_val2 |= (1<<28);
			}
		}
		
		MPWMx->CAPCNTV = reg_val;
		MPWMx->SCAPV = reg_val2;
	}
	else if (pha_val == PH_W)
	{
		reg_val = MPWMx->CAPCNTW;
		reg_val2 = MPWMx->SCAPW;
		
		reg_val &= ~(1<<27);
		reg_val2 &= ~(1<<28);
		
		if (NewStatus == ENABLE)
		{
			reg_val |= (1<<27);
			
			if (ScapEdge == SUBCAP_RISING)
			{
				reg_val2 |= (0<<28);
			}
			else if (ScapEdge == SUBCAP_FALLING)
			{
				reg_val2 |= (1<<28);
			}
		}
		
		MPWMx->CAPCNTW = reg_val;
		MPWMx->SCAPW = reg_val2;
	}
	return HAL_OK;
}


/********************************************************************//**
 * @brief 		Capture Counter Clear
 * @param[in]	MPWMx	MPWM peripheral selected, should be:
 * 					- MPWMx	:MPWM0~1 peripheral
 * @param[in]	pha_val			phase selection
 * 					- 0 = PH_U 
 * 					- 1 = PH_V
 * 					- 2 = PH_W
 * @param[in]	CapMode Phase Value
 *						- MPWM_MAIN_CAP_CNT
 *						- MPWM_RISING_CAP_VAL
 *						- MPWM_FALLING_CAP_VAL
 * 					- MPWM_SUB_CAP_VAL
 * @return 		HAL Status
 * 					- HAL_OK
 * 					- HAL_ERROR
 *********************************************************************/
HAL_Status_Type HAL_MPWM_ExtClearCaptureCmd(MPWM_Type *MPWMx, MPWM_PH_VAL pha_val, MPWM_CAP_MODE CapMode)
{
	/* Check MPWM  handle */
         if(MPWMx == NULL)
        {
            return HAL_ERROR;
         }

	
	if (pha_val == PH_U)
	{
		if (CapMode == MPWM_MAIN_CAP_CNT)
		{
			MPWMx->CAPCNTU |= (1UL<<31);
		}
		else if (CapMode == MPWM_RISING_CAP_VAL)
		{
			MPWMx->RCAPU |= (1UL<<31);
		}
		else if (CapMode == MPWM_FALLING_CAP_VAL)
		{
			MPWMx->FCAPU |= (1UL<<31);
		}
		else if (CapMode == MPWM_SUB_CAP_VAL)
		{
			MPWMx->SCAPU |= (1UL<<31);
		}
	}
	else if (pha_val == PH_V)
	{
		if (CapMode == MPWM_MAIN_CAP_CNT)
		{
			MPWMx->CAPCNTV |= (1UL<<31);
		}
		else if (CapMode == MPWM_RISING_CAP_VAL)
		{
			MPWMx->RCAPV |= (1UL<<31);
		}
		else if (CapMode == MPWM_FALLING_CAP_VAL)
		{
			MPWMx->FCAPV |= (1UL<<31);
		}
		else if (CapMode == MPWM_SUB_CAP_VAL)
		{
			MPWMx->SCAPV |= (1UL<<31);
		}
	}
	else if (pha_val == PH_W)
	{
		if (CapMode == MPWM_MAIN_CAP_CNT)
		{
			MPWMx->CAPCNTW |= (1UL<<31);
		}
		else if (CapMode == MPWM_RISING_CAP_VAL)
		{
			MPWMx->RCAPW |= (1UL<<31);
		}
		else if (CapMode == MPWM_FALLING_CAP_VAL)
		{
			MPWMx->FCAPW |= (1UL<<31);
		}
		else if (CapMode == MPWM_SUB_CAP_VAL)
		{
			MPWMx->SCAPW |= (1UL<<31);
		}
	}
	return HAL_OK;
}		

/********************************************************************//**
 * @brief 		Get Capture Counter Value
 * @param[in]	MPWMx	MPWM peripheral selected, should be:
 * 					- MPWMx	:MPWM0~1 peripheral
 * @param[in]	pha_val			phase selection
 * 					- 0 = PH_U 
 * 					- 1 = PH_V
 * 					- 2 = PH_W
 * @return 		Capture Counter Value
 *********************************************************************/
uint32_t HAL_MPWM_ExtGetCaptureCNT(MPWM_Type *MPWMx, MPWM_PH_VAL pha_val)
{
	/* Check MPWM  handle */
         if(MPWMx == NULL)
        {
            return HAL_ERROR;
         }

	if (pha_val == PH_U)
	{
		return (MPWMx->CAPCNTU & 0x1FFFF);
	}
	else if (pha_val == PH_V)
	{
		return (MPWMx->CAPCNTV & 0x1FFFF);
	}
	else if (pha_val == PH_W)
	{
		return (MPWMx->CAPCNTW & 0x1FFFF);
	}
	
	return 0;
}


/********************************************************************//**
 * @brief 		Get Capture Value
 * @param[in]	MPWMx	MPWM peripheral selected, should be:
 * 					- MPWMx	:MPWM0~1 peripheral
 * @param[in]	pha_val			phase selection
 * 					- 0 = PH_U 
 * 					- 1 = PH_V
 * 					- 2 = PH_W
 * @param[in]	CapSel			capture mode selection
 *						-0 = RISING_CAP_VAL
 *						-1 = FALLING_CAP_VAL
 *						-2 = SUB_CAP_VAL
 * @return 		Capture Value
 *********************************************************************/
uint32_t HAL_MPWM_ExtGetCaptureVal(MPWM_Type *MPWMx, MPWM_PH_VAL pha_val, MPWM_CAP_SEL CapSel)
{

	if (pha_val == PH_U)
	{
		if (CapSel == RISING_CAP_VAL)
		{
			return (MPWMx->RCAPU & 0x1FFFF);
		}
		else if (CapSel == FALLING_CAP_VAL)
		{
			return (MPWMx->FCAPU & 0x1FFFF);
		}
		else if (CapSel == SUB_CAP_VAL)
		{
			return (MPWMx->SCAPU & 0x1FFFF);
		}
	}
	else if (pha_val == PH_V)
	{
		if (CapSel == RISING_CAP_VAL)
		{
			return (MPWMx->RCAPV & 0x1FFFF);
		}
		else if (CapSel == FALLING_CAP_VAL)
		{
			return (MPWMx->FCAPV & 0x1FFFF);
		}
		else if (CapSel == SUB_CAP_VAL)
		{
			return (MPWMx->SCAPV & 0x1FFFF);
		}
	}
	else if (pha_val == PH_W)
	{
		if (CapSel == RISING_CAP_VAL)
		{
			return (MPWMx->RCAPW & 0x1FFFF);
		}
		else if (CapSel == FALLING_CAP_VAL)
		{
			return (MPWMx->FCAPW & 0x1FFFF);
		}
		else if (CapSel == SUB_CAP_VAL)
		{
			return (MPWMx->SCAPW & 0x1FFFF);
		}
	}
	
	return 0;
}


/********************************************************************//**
 * @brief		Initializes the MPWMx peripheral in Extend Mode
 *               parameters.
 * @param[in]	MPWMx	MPWM peripheral selected, should be:
 * 					- MPWMx	:MPWM0~1 peripheral
 * @return 		HAL Status
 * 					- HAL_OK
 * 					- HAL_ERROR
 *********************************************************************/
HAL_Status_Type HAL_MPWM_ExtInit(MPWM_Type *MPWMx)
{
	/* Check MPWM  handle */
         if(MPWMx == NULL)
        {
            return HAL_ERROR;
         }

	SYST_ACCESS_EN();
	
	if(MPWMx == MPWM0)
	{
		SCU->PER2&=~(1UL<<16);
		SCU->PCER2&=~(1UL<<16);
		
		/* Set up peripheral clock for MPWM0 module */
		SCU->PER2|=(1UL<<16);
		SCU->PCER2|=(1UL<<16);
	}
	else if(MPWMx == MPWM1)
	{
		SCU->PER2&=~(1UL<<17);
		SCU->PCER2&=~(1UL<<17);
		
		/* Set up peripheral clock for MPWM1 module */
		SCU->PER2|=(1UL<<17);
		SCU->PCER2|=(1UL<<17);
	}
	
	SYST_ACCESS_DIS();
	
	// Extend Mode
	MPWMx->MR &= ~(0x03<<14);
	MPWMx->MR |= (0x03<<14);
	
	// PWMEN bit = '1'
	MPWMx->CR3 = 0
	|(0UL<<20)						// Ph-W IRQN[2:0]
	|(1UL<<16)						// Ph-W PWMEN
	|(0UL<<12)						// Ph-V IRQN[2:0]
	|(1UL<<8)							// Ph-V PWMEN
	|(0UL<<4)							// Ph-U IRQN[2:0]
	|(1UL<<0)							// Ph-U PWMEN
	;
	
	MPWMx->CR4 = 0;					// Ph-U, V, W PSTART Disable
	MPWMx->OLR = 0;         			// MPWM Output Level 
	MPWMx->FOLR = 0;       			// MPWM Force Output Level
	MPWMx->PCR = 0;        			// Protection 0 Control 
	MPWMx->PSR = (0xca<<8) | 0;      // Protection 0 Status  
	MPWMx->OCR = 0;        			// Protection 1 Control 
	MPWMx->OSR = (0xac<<8) | 0;      // Protection 1 Status  
	MPWMx->ATR1 = 0;
	MPWMx->ATR2 = 0;
	MPWMx->ATR3 = 0;
	MPWMx->ATR4 = 0;
	MPWMx->ATR5 = 0;
	MPWMx->ATR6 = 0;
	return HAL_OK;

}


/*********************************************************************//**
 * @brief		De-initializes the MPWMx peripheral registers to their
 *              default reset values.
 * @param[in]	MPWMx	MPWM peripheral selected, should be:
 * 					- MPWMx	:MPWM0~1 peripheral
 * @return 		HAL Status
 * 					- HAL_OK
 * 					- HAL_ERROR
 **********************************************************************/
HAL_Status_Type HAL_MPWM_ExtDeInit(MPWM_Type* MPWMx)
{
	/* Check MPWM  handle */
  if(MPWMx == NULL)
  {
     return HAL_ERROR;
   }

	MPWMx->CR4 = 0
	| (1<<16)
	| (1<<8)
	| (1<<0)
	; 				// disable PWMEN
	
	SYST_ACCESS_EN();
	
	if (MPWMx == MPWM0)
	{
		SCU->PER2&=~(1UL<<16);
		SCU->PCER2&=~(1UL<<16);
	}
	else if(MPWMx == MPWM1)
	{
		SCU->PER2&=~(1UL<<17);
		SCU->PCER2&=~(1UL<<17);
	}
	
	SYST_ACCESS_DIS();
	return HAL_OK;
}


/**********************************************************************//**
 * @brief		Set PWM Comparator Protection 
 * @param[in]	MPWMx	MPWM peripheral selected, should be:
 * 					- MPWMx	:MPWM0~1 peripheral
 * @param[in]	pha_val			phase selection
 * 					- 0 = PH_U 
 * 					- 1 = PH_V
 * 					- 2 = PH_W
 * @param[in]	comp	Comparator Select
 *					- 0 ~ 3
 * @param[in]	NewState New State of MPWMx peripheral's operation, should be:
 * 					- ENABLE
 * 					- DISABLE
 * @return 		HAL Status
 * 					- HAL_OK
 * 					- HAL_ERROR
 **********************************************************************/
HAL_Status_Type HAL_MPWM_SetCMPProt(MPWM_Type* MPWMx, MPWM_PH_VAL pha_val, uint32_t comp, FunctionalState state)
{
	uint32_t reg_val;

	/* Check MPWM  handle */
  if(MPWMx == NULL)
  {
     return HAL_ERROR;
  }
	
	if (pha_val == PH_W)
	{
			 reg_val = MPWMx->CPCR & ~(0x7<<27);
			 reg_val |= (comp<<28);
		if(state==ENABLE)
		{
			reg_val |= 1<<27;
		}
	}
	else if (pha_val == PH_V)
	{
		  reg_val = MPWMx->CPCR & ~(0x7<<19);
			reg_val |= (comp<<20);
		if(state==ENABLE)
		{
			reg_val |= 1<<19;
		}
	}
	else if (pha_val == PH_U)
	{
		  reg_val = MPWMx->CPCR & ~(0x7<<11);
			reg_val |= (comp<<12);
		if(state==ENABLE)
		{
			reg_val |= 1<<11;
		}
	}
	
	MPWMx->CPCR = reg_val;
	return HAL_OK;
}

/* --------------------------------- End Of File ------------------------------ */

