/**********************************************************************//**
* @file		A33M11x_hal_scu.c
* @brief	Contains all functions support for SCU firmware library on A33M11x
* @version	1.01
* @date		
* @author	ABOV PTS team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
************************************************************************
* ABOV Disclaimer
*
*IMPORTANT NOTICE ? PLEASE READ CAREFULLY
*ABOV Semiconductor ("ABOV") reserves the right to make changes, corrections, enhancements, 
*modifications, and improvements to ABOV products and/or to this document at any time without notice. 
*ABOV does not give warranties as to the accuracy or completeness of the information included herein.
*Purchasers should obtain the latest relevant information of ABOV products before placing orders. 
*Purchasers are entirely responsible for the choice, selection, and use of ABOV products and 
*ABOV assumes no liability for application assistance or the design of purchaser products. No license, 
*express or implied, to any intellectual property rights is granted by ABOV herein. 
*ABOV disclaims all express and implied warranties and shall not be responsible or
*liable for any injuries or damages related to use of ABOV products in such unauthorized applications. 
*ABOV and the ABOV logo are trademarks of ABOV.
*All other product or service names are the property of their respective owners. 
*Information in this document supersedes and replaces the information previously
*supplied in any former versions of this document.
*2020 ABOV Semiconductor  All rights reserved
*
**********************************************************************/

/* Includes ------------------------------------------------------------------- */
#include "A33M11x_hal_scu.h"
#include "A33M11x_hal_cfmc.h"

/**********************************************************************//**
 * @brief 	  monitoring External oscillator  
 * @param  none
 * @return	status enumeration value:
 *              - ERROR
 *              - SUCCESS
 **********************************************************************/
Status HAL_SCU_HSE_ClockMonitoring(void)
{
	__IO uint32_t StartUpCounter = 0;
	__IO uint16_t status;
	uint32_t i;

	SYST_ACCESS_EN();
	// monitoring External oscillator 
	SCU->CMR|=(1<<3); // 0:External oscillator monitoring disabled, 1:External oscillator monitoring enable

	SYST_ACCESS_DIS();
	
	/* need more Wait for MOSC stable, check XOUT pin oscillation  */	
	for (i=0;i<0x800;i++)
	{
           __NOP();
	}
	
	if(SCU->CMR&(1<<0)) return ERROR;
	else return SUCCESS;
}



/**********************************************************************//**
 * @brief 		System Mode Register  
 * @param[in]	operating clk	
 * 					- SCU_SMR_HSEAON : (1<<12)
 * 					- SCU_SMR_PLLAON  : (1<<11)
 * 					- SCU_SMR_HSIAON : (1<<10)
 *					- SCU_SMR_LSIAON : (1<<9)
 *					- SCU_SMR_VDCAON  : (1<<8)
 * @param[in]	NewState
 *					- ENABLE : Bit Enable
 * 				- DISABLE : Bit Disable
 * @return		None
 **********************************************************************/
void HAL_SCU_SMRCmd(uint16_t operating_clk, FunctionalState NewState)
{
	uint16_t	reg_val;
	
	SYST_ACCESS_EN();
	
	reg_val = SCU->SMR;
	reg_val &= ~(operating_clk);

	if (NewState == ENABLE)
	{
		reg_val |= (operating_clk);
	}
	
	SCU->SMR = reg_val;
	
	SYST_ACCESS_DIS();
}



/**********************************************************************//**
 * @brief 		Prev Mode Register  
 * @param[in]	None 
 * @return		Pervious mode bit
 * 					- 0 : Run Mode
 * 					- 1 : Sleep Mode
 * 					- 2 : Deep-Sleep Mode
 * 					- 3 : INIT Mode
 **********************************************************************/
uint16_t HAL_SCU_GetPREVStatus (void)
{
	uint16_t	reg_val;
	
	reg_val = (((SCU->SMR)>>4) & 0x03);
	
	return reg_val;
}



/**********************************************************************//**
 * @brief 		Wake Up Source Enable/Disable  
 * @param[in]	WakeUpSrc
 * 					- SYSTICKUE : (1<<16)
 * 					- GPIOGWUE : (1<<14)
 * 					- GPIOFWUE : (1<<13)
 * 					- GPIODWUE : (1<<11)
 *						- GPIOCWUE : (1<<10)
 * 					- GPIOBWUE : (1<<9)
 * 					- GPIOAWUE : (1<<8)
 * 					- FRTWUE  : (1<<2)
 *						- WDTWUE   : (1<<1)
 * 					- LVIWUE   : (1<<0)
 *
 * @param[in]	NewState
 *					- ENABLE : Bit Enable
 * 				- DISABLE : Bit Disable
 * 
 * @return		None
 **********************************************************************/
void HAL_SCU_WakeUpSRCCmd(uint32_t WakeUpSrc, FunctionalState NewState)
{
	uint32_t		reg_val;
	
	SYST_ACCESS_EN();
	
	reg_val = SCU->WUER;
	reg_val &= ~(WakeUpSrc);
	
	if (NewState == ENABLE)
	{
		reg_val |= (WakeUpSrc);
	}
	
	SCU->WUER = reg_val;
	
	SYST_ACCESS_DIS();
}

/*********************************************************************//**
 * @brief 		Get current value of WakeUpSRC Status
 * @param[in]	None
 * @return		Current value of WakeUpSRC Status register
 *********************************************************************/
uint16_t HAL_SCU_GetWakeUpSRCStatus(void)
{
	return (SCU->WUSR);
}



/**********************************************************************//**
 * @brief 		Reset Source Enable/Disable Register  
 * @param[in]	ResetSRC
 * 					- RST_LOCKUPRST : (1<<9)
 * 					- RST_PORST    : (1<<8)
 * 					- RST_PINRST    : (1<<7)
 * 					- RST_CPURST     : (1<<6)
 *						- RST_SWRST    : (1<<5)
 * 					- RST_WDTRST   : (1<<4)
 * 					- RST_MCLKFRST    : (1<<3)
 * 					- RST_HSEFRST    : (1<<1)
 *						- RST_LVDRST  : (1<<0)
 * @param[in]	NewState
 *						- ENABLE : Bit Enable
 * 					- DISABLE : Bit Disable
 * @return		None
 **********************************************************************/
void HAL_SCU_ResetSRCCmd(uint16_t ResetSRC, FunctionalState NewState)
{
	uint16_t		reg_val;
	
	SYST_ACCESS_EN();
	
	reg_val = SCU->RSER;
	reg_val &= ~(ResetSRC);
	
	if (NewState == ENABLE)
	{
		reg_val |= (ResetSRC);
	}
	
	SCU->RSER = reg_val;
	
	SYST_ACCESS_DIS();
}




/*********************************************************************//**
 * @brief 		Get current value of Reset Source Status.
 * @param[in]	None
 * @return		Current value of ResetSRC Status register.
 *********************************************************************/
uint16_t HAL_SCU_GetResetSRCStatus(void)
{
	return (SCU->RSSR);
}



/**********************************************************************//**
 * @brief 		Peripheral Enable/Disable Register  
 * @param[in]	PeriSRC
 * 				- PRRI_QEI1    : (1<<29)
 * 				- PERI_QEI0    : (1<<28)
 * 				- PERI_TIMER7  : (1<<23)
 * 				- PERI_TIMER6  : (1<<22)
 * 				- PERI_TIMER5  : (1<<21)
 * 				- PERI_TIMER4  : (1<<20)
 * 				- PERI_TIMER3  : (1<<19)
 * 				- PERI_TIMER2  : (1<<18)
 * 				- PERI_TIMER1  : (1<<17)
 * 				- PERI_TIMER0  : (1<<16)
 *					- PERI_GPIOG   : (1<<14)
 *					- PERI_GPIOF   : (1<<13)
 *					- PERI_GPIOD   : (1<<11)
 *					- PERI_GPIOC   : (1<<10)
 *					- PERI_GPIOB   : (1<<9)
 *					- PERI_GPIOA   : (1<<8)
 * 				- PERI_FRT    : (1<<7)
 * 				- PERI_DMA     : (1<<4)
 *
 * @param[in]	NewState
 *					- ENABLE : Bit Enable
 * 				- DISABLE : Bit Disable
 * 
 * @return		None
 **********************************************************************/
void HAL_SCU_PeriSRCCmd_1(uint32_t PeriSRC, FunctionalState NewState)
{
	uint32_t		reg_val;
	
	SYST_ACCESS_EN();
	
	reg_val = SCU->PER1;
	reg_val &= ~(PeriSRC);
	
	if (NewState == ENABLE)
	{
		reg_val |= (PeriSRC);
	}
	
	SCU->PER1 = reg_val;
	
	SYST_ACCESS_DIS();
}


/**********************************************************************//**
 * @brief 		Peripheral Enable/Disable Register  
 * @param[in]	PeriSRC
 * 					- PERI_CRC : (1<<29)
 * 					- PERI_AFE : (1<<24)
 * 					- PERI_ADC1       : (1<<21)
 * 					- PERI_ADC0       : (1<<20)
 * 					- PERI_MPWM1      : (1<<17)
 * 					- PERI_MPWM0      : (1<<16)
 * 					- PERI_UART3      : (1<<11)
 *						- PERI_UART2      : (1<<10)
 *						- PERI_UART1      : (1<<9)
 *						- PERI_UART0      : (1<<8)
 *						- PERI_I2C1       : (1<<5)
 *						- PERI_I2C0       : (1<<4)
 * 					- PERI_SPI1       : (1<<1)
 * 					- PERI_SPI0       : (1<<0)
 *
 * @param[in]	NewState
 *					- ENABLE : Bit Enable
 * 					- DISABLE : Bit Disable
 * 
 * @return		None
 **********************************************************************/
void HAL_SCU_PeriSRCCmd_2(uint32_t PeriSRC, FunctionalState NewState)
{
	uint32_t		reg_val;
	
	SYST_ACCESS_EN();
	
	reg_val = SCU->PER2;
	reg_val &= ~(PeriSRC);
	
	if (NewState == ENABLE)
	{
		reg_val |= (PeriSRC);
	}
	
	SCU->PER2 = reg_val;
	
	SYST_ACCESS_DIS();
}


/**********************************************************************//**
 * @brief 		Peripheral Clock Enable/Disable Register  
 * @param[in]	PeriSRC
 * 				- PERI_QEI1    : (1<<29)
 * 				- PERI_QEI0    : (1<<28)
 * 				- PERI_TIMER7  : (1<<23)
 * 				- PERI_TIMER6  : (1<<22)
 * 				- PERI_TIMER5  : (1<<21)
 * 				- PERI_TIMER4  : (1<<20)
 * 				- PERI_TIMER3  : (1<<19)
 * 				- PERI_TIMER2  : (1<<18)
 * 				- PERI_TIMER1  : (1<<17)
 * 				- PERI_TIMER0  : (1<<16)
 *					- PERI_GPIOG   : (1<<14)
 *					- PERI_GPIOF   : (1<<13)
 *					- PERI_GPIOD   : (1<<11)
 *					- PERI_GPIOC   : (1<<10)
 *					- PERI_GPIOB   : (1<<9)
 *					- PERI_GPIOA   : (1<<8)
 * 				- PERI_FRT    : (1<<6)
 * 				- PERI_DMA     : (1<<4)
 *
 * @param[in]	NewState
 *					- ENABLE : Bit Enable
 * 				- DISABLE : Bit Disable
 * 
 * @return		None
 **********************************************************************/
void HAL_SCU_PeriCLKCmd_1(uint32_t PeriSRC, FunctionalState NewState)
{
	uint32_t		reg_val;
	
	SYST_ACCESS_EN();
	
	reg_val = SCU->PCER1;
	reg_val &= ~(PeriSRC);
	
	if (NewState == ENABLE)
	{
		reg_val |= (PeriSRC);
	}
	
	SCU->PCER1 = reg_val;
	
	SYST_ACCESS_DIS();
}


/**********************************************************************//**
 * @brief 		Peripheral Clock Enable/Disable Register  
 * @param[in]	PeriSRC
 * 				- PERI_CRC      : (1<<24)
 * 				- PERI_AFE      : (1<<24)
 * 				- PERI_ADC1       : (1<<21)
 * 				- PERI_ADC0       : (1<<20)
 * 				- PERI_MPWM1      : (1<<17)
 * 				- PERI_MPWM0      : (1<<16)
 * 				- PERI_UART3      : (1<<11)
 *					- PERI_UART2      : (1<<10)
 *					- PERI_UART1      : (1<<9)
 *					- PERI_UART0      : (1<<8)
 *					- PERI_I2C1       : (1<<5)
 *					- PERI_I2C0       : (1<<4)
 * 				- PERI_SPI1       : (1<<1)
 * 				- PERI_SPI0       : (1<<0)
 * @param[in]	NewState
 *					- ENABLE : Bit Enable
 * 				- DISABLE : Bit Disable
 * 
 * @return		None
 **********************************************************************/
void HAL_SCU_PeriCLKCmd_2(uint32_t PeriSRC, FunctionalState NewState)
{
	uint32_t		reg_val;
	
	SYST_ACCESS_EN();
	
	reg_val = SCU->PCER2;
	reg_val &= ~(PeriSRC);
	
	if (NewState == ENABLE)
	{
		reg_val |= (PeriSRC);
	}
	
	SCU->PER2 = reg_val;
	
	SYST_ACCESS_DIS();
}



/**********************************************************************//**
 * @brief 		Clock Source Control Register  
 * @param[in]	ClockSRC clock Function mode, should be:
 * 					- SCU_CSCR_HSE_ON   	   : (1<<5)
 * 					- SCU_CSCR_HSI_ON    	   : (1<<3)
 * 					- SCU_CSCR_LSI_ON 		   : (1<<1)
 * @param[in]	NewState
 *					- Enable : Bit Enable
 * 					- Disable : Bit Disable
 * @return		None
 **********************************************************************/
void HAL_SCU_ClockSRCCmd (uint8_t ClockSRC, FunctionalState NewState)
{
	uint32_t		reg_val, i;
	
	SYST_ACCESS_EN();
	
	reg_val = SCU->CSCR;
	
	reg_val &= ~ ClockSRC;

	if (NewState == ENABLE)
	{
		reg_val |= (ClockSRC);
	}
	
	SCU->CSCR = reg_val;
		
	// Stable Time
	for(i=0; i<1000; i++);
	
	SYST_ACCESS_DIS();
}


/**********************************************************************//**
 * @brief 		PCLK Input Clock Divider Register  
 * @param[in]	PCLKDIV	clock Function mode, should be:
 * 					- PCLKDIV_HCLK_1   : (0)
 * 					- PCLKDIV_HCLK_2 : (1)
 * 					- PCLKDIV_HCLK_4  : (2)
 * 					- PCLKDIV_HCLK_8  : (3)
 * 					- PCLKDIV_HCLK_16 : (4)
 *
 * @return		None
 **********************************************************************/
void HAL_SCU_PCLKDIVCmd (uint8_t PCLKDIV)
{
	uint32_t		reg_val;
	
	SYST_ACCESS_EN();
	
	reg_val = SCU->SCCR;
	
	reg_val &= ~SCU_SCCR_PCLKDIV_MSK;

	reg_val |= (PCLKDIV<<16);
	
	SCU->SCCR = reg_val;
	
	SYST_ACCESS_DIS();
}


/**********************************************************************//**
 * @brief 		PLL Input Clock Selection  
 * @param[in]	PLLINSRC	clock Function mode, should be:
 * 					- SCU_SCCR_PLLCLKSEL_HSI  : (0<<12)
 * 					- SCU_SCCR_PLLCLKSEL_HSE  : (1<<12)
 *
 * @return		None
 **********************************************************************/
void HAL_SCU_PLLINCLKCmd (uint32_t PLLINSRC)
{
	uint32_t		reg_val;
	
	SYST_ACCESS_EN();
	
	reg_val = SCU->SCCR;
	
	reg_val &= ~(SCU_SCCR_PLLCLKSEL_MSK);

	reg_val |= (PLLINSRC&SCU_SCCR_PLLCLKSEL_MSK);
	
	SCU->SCCR = reg_val;
	
	SYST_ACCESS_DIS();
}


/**********************************************************************//**
 * @brief 		PLL Input Clock Divider  
 * @param[in]	PLLPREDIV	clock Function mode, should be:
 * 					- PLLPREDIV_PLLINCLK_1 =PLLCLKSEL/1	   : (0)
 * 					- PLLPREDIV_PLLINCLK_2 =PLLCLKSEL/2	   : (1)
 * 					- PLLPREDIV_PLLINCLK_4 =PLLCLKSEL/4	   : (2)
 * 					- PLLPREDIV_PLLINCLK_8 =PLLCLKSEL/8	   : (3)
 *
 * @return		None
 **********************************************************************/
void HAL_SCU_PLLPREDIVCmd (uint8_t PLLPREDIV)
{
	uint32_t		reg_val;
	
	SYST_ACCESS_EN();
	
	reg_val = SCU->SCCR;
	
	reg_val &= ~(SCU_SCCR_PLLPREDIV_MSK);

	reg_val |= ((PLLPREDIV)<<8);
	
	SCU->SCCR = reg_val;
	
	SYST_ACCESS_DIS();
}

/**********************************************************************//**
 * @brief 		HCLK Divider Selection  
 * @param[in]	HCLKDIV	clock Function mode, should be:
 * 					- HCLKDIV_MCLK_1		   : (0)
 * 					- HCLKDIV_MCLK_2	   : (1)
 * 					- HCLKDIV_MCLK_4	   : (2)
 * 					- HCLKDIV_MCLK_8   : (3)
 * 					- HCLKDIV_MCLK_16   : (4)
 * 					- HCLKDIV_MCLK_32   : (5)
 * 					- HCLKDIV_MCLK_64  : (6)
 * 					- HCLKDIV_MCLK_128	   : (7)
 * 					- HCLKDIV_MCLK_256	   : (8)
 * 					- HCLKDIV_MCLK_512	   : (9)
 *
 * @return		None
 **********************************************************************/
void HAL_SCU_HCLKDIVCmd (uint8_t HCLKDIV)
{
	uint32_t		reg_val;
	
	SYST_ACCESS_EN();
	
	reg_val = SCU->SCCR;
	
	reg_val &= ~(SCU_SCCR_HCLKDIV_MSK);

	reg_val |= ((HCLKDIV&0xF)<<24);
	
	SCU->SCCR = reg_val;
	
	SYST_ACCESS_DIS();
}


/**********************************************************************//**
 * @brief 		System Clock Selection  
 * @param[in]	MCLKSRC	clock Function mode, should be:
 * 					- SCU_SCCR_MCLKSEL_LSI		   	: (0)
 * 					- SCU_SCCR_MCLKSEL_HSI			: (1)
 * 					- SCU_SCCR_MCLKSEL_PLL			: (2)
 * 					- SCU_SCCR_MCLKSEL_HSE		   	: (3)
 *
 * @return		None
 **********************************************************************/
void HAL_SCU_SystemClockChange (uint8_t MCLKSRC)
{
	uint32_t		reg_val;
	
	SYST_ACCESS_EN();
	
	reg_val = SCU->SCCR;
	
	reg_val &= ~(SCU_SCCR_MCLKSEL_MSK);

	reg_val |= (MCLKSRC&SCU_SCCR_MCLKSEL_MSK);
	
	SCU->SCCR = reg_val;
	
	SYST_ACCESS_DIS();
}



/**********************************************************************//**
 * @brief 		Clock Monitoring Register  
 * @param[in]	MonioringSRC	clock Function mode, should be:
 * 					- SCU_CMR_MCLKREC		: (1<<15)
 * 					- SCU_CMR_MCLKMNT		: (1<<7)
 * 					- SCU_CMR_MCLKIE		: (1<<6)
 * 					- SCU_CMR_HSEMNT		: (1<<3)
 * 					- SCU_CMR_HSEIE		: (1<<2)
 * @param[in]	NewState
 *						- ENABLE : Bit Enable
 * 					- DISABLE : Bit Disable
 * @return		None
 **********************************************************************/
void HAL_SCU_ClockMonitoringCmd (uint32_t MonioringSRC, FunctionalState NewState)
{
	uint32_t		reg_val;
	
	SYST_ACCESS_EN();
	
	reg_val = SCU->CMR;
	
	reg_val &= ~(MonioringSRC&0x8CCC);

	if (NewState == ENABLE)
	{
		reg_val |= (MonioringSRC&0x8CCC);
	}
	
	SCU->CMR = reg_val;
	
	SYST_ACCESS_DIS();
}


/**********************************************************************//**
 * @brief 		Clock Monitoring Flag Clear  
 * @param[in]	ClearMNTSRC	clock Function mode, should be:
 * 					- SCU_CMR_MCLKFAIL		: (1<<5)
 * 					- SCU_CMR_HSEFAIL		: (1<<1)
 *
 * @return		None
 **********************************************************************/
void HAL_SCU_ClearCLKMNTFlg (uint8_t ClearMNTSRC)
{
	SYST_ACCESS_EN();
	
	SCU->CMR |= (ClearMNTSRC&0x22);

	SYST_ACCESS_DIS();
}


/*********************************************************************//**
 * @brief 		Get current value of Clock Monitoring Status.
 * @param[in]	None
 * @return		Current value of Clock Monitoring Status register.
 *********************************************************************/
uint16_t HAL_SCU_GetCLKMNTStatus(void)
{
	return (SCU->CMR);
}


/**********************************************************************//**
 * @brief 		NMI Control Register  
 * @param[in]	NMI Source selection, should be:
 * 					- NMISRC : 0x00~0xFF
 * @param[in]	NMI Interrupt selection, should be:
 * 					- SCU_NMICR_NMIINEN		: (1<<15)
 * 					- SCU_NMICR_PROT1EN		: (1<<6)
 * 					- SCU_NMICR_OVP1EN		: (1<<5)
 * 					- SCU_NMICR_PROT0EN		: (1<<4)
 * 					- SCU_NMICR_OVP0EN		: (1<<3)
 * 					- SCU_NMICR_WDTINTEN	: (1<<2)
 * 					- SCU_NMICR_MCLKFAILEN	: (1<<1)
 * 					- SCU_NMICR_LVIEN		: (1<<0)
 * @param[in]	NewState
 *						- ENABLE : Bit Enable
 * 					- DISABLE : Bit Disable
 * @return		None
 **********************************************************************/
void HAL_SCU_SetNMI (uint8_t NMISRC, uint16_t NMIINTB, FunctionalState NewState)
{
	uint32_t		reg_val;
	
	SYST_ACCESS_EN();
	
	reg_val = SCU->NMICR;
	
	reg_val &= ~(NMIINTB);
	
	if(NMIINTB == SCU_NMICR_NMIINEN)
	{
		reg_val &= ~((0xFF<<16));
		reg_val |= ((NMISRC&0xFF)<<16);
	}
	
	if (NewState == ENABLE)
	{
		reg_val |= (NMIINTB);
	}
	
	SCU->NMICR = reg_val;
	
	SYST_ACCESS_DIS();
}


/*********************************************************************//**
 * @brief 		Get current value of NMI Status.
 * @param[in]	None.
 * @return		Current value of NMI Status register.
 *********************************************************************/
uint32_t HAL_SCU_GetNMIStatus(void)
{
	return (SCU->NMISR);
}


/**********************************************************************//**
 * @brief 		NMI Flag Clear  
 * @param[in]	ClearNMISRC	clock Function mode, should be:
 * 					- SCU_NMISR_NMIINTSTS	: (1<<15)
 * 					- SCU_NMISR_PROT1		: (1<<6)
 * 					- SCU_NMISR_OVP1		: (1<<5)
 * 					- SCU_NMISR_PROT0		: (1<<4)
 * 					- SCU_NMISR_OVP0		: (1<<3)
 * 					- SCU_NMISR_WDTINT		: (1<<2)
 * 					- SCU_NMISR_MCLKFAIL		: (1<<1)
 * 					- SCU_NMISR_LVISTS		: (1<<0)
 * @return		None
 **********************************************************************/
void HAL_SCU_ClearNMIFlg (uint16_t ClearNMISRC)
{
	SCU->NMISR |= (SCU_NMISR_WTIDKY | (ClearNMISRC&0x7F));
}


/**********************************************************************//**
 * @brief 		Clock Output Register  
* @param[in]	CLKOSRC	Clock source selection, should be:
 * 					- SCU_COR_CLKOINSEL_LSI : (0<<5)
 * 					- SCU_COR_CLKOINSEL_MCLK : (4<<5)
 * 					- SCU_COR_CLKOINSEL_HSI : (5<<5)
 * 					- SCU_COR_CLKOINSEL_HSE : (6<<5)
 * 					- SCU_COR_CLKOINSEL_PLL : (7<<5)
 * @param[in]	CLKODIV	Clock divider, should be:
 * 					- 0x0 ~ 0xF
 * @param[in]	NewState
 *					- ENABLE : Bit Enable
 * 				- DISABLE : Bit Disable
 * @return		None
 **********************************************************************/
void HAL_SCU_ClockOutput (uint8_t CLKOSRC, uint8_t CLKODIV, FunctionalState NewState)
{
	uint32_t		reg_val;
	
	SYST_ACCESS_EN();
	
	reg_val = SCU->COR;
	
	reg_val = 0;
	
	reg_val |= ((CLKOSRC&(0x07<<5)) | (CLKODIV&0x0F));
	
	if (NewState == ENABLE)
	{
		reg_val |= (SCU_COR_CLKOEN_ENABLE);
	}
	
	SCU->COR = reg_val;
	
	SYST_ACCESS_DIS();
}


/**
 * @brief 	 monitoring External oscillator  
 * @param 	 none
 * @return	 status 	enumeration value:
 *              - ERROR
 *              - SUCCESS

PLL_DEALY value must be changed according to the system clock used by the user.
Set to about 200us based on the system clock.
1. LSI500kHz		: 10 
2. HSE8MHz : 160
3. HSE16MHz	: 320
4. HSI32MHz		: 640


*/
#define PLL_DELAY	160	// @HSE 8MHz

/**********************************************************************//**
 * @brief 		PLL Control Register  
 * @param[in]	clock Function mode, should be:
 * 					
 *								Fin*(N1+1)
 *					Fout = --------------------- x D
 *							(R+1)*(N2+1)*(P+1)
 *
 *					-> List
 * 					  D  = PLLMODE  [20:20]
 * 					  R  = PREDIV   [18:16]
 *			 		  N1 = POSTDIV1 [15:08]
 *					  N2 = POSTDIV2 [07:04]
 *					  P  = OUTDIV   [03:00]
 * @return		None
 **********************************************************************/
Status HAL_SCU_PLL_ClockConfig (uint32_t PLLMODE, uint8_t PREDIV, uint8_t POSTDIV1, uint8_t POSTDIV2, uint8_t OUTDIV, FunctionalState NewState)
{	
	__IO uint32_t StartUpCounter = 0; 
	__IO uint32_t reg_val;
	__IO uint32_t i;
	
	SYST_ACCESS_EN();
	
	if (NewState == DISABLE){
		SCU->PLLCON =0;
		return SUCCESS;
	}
	else {
		SCU->PLLCON |= (SCU_PLLCON_PLLEN);	// PLLEN	
		__NOP();	//1us wait
		
		reg_val = SCU->PLLCON;
		
		reg_val &= ~(0x001FFFFF);
		
		reg_val |= (SCU_PLLCON_PLLRSTB) |(SCU_PLLCON_BYPASSB) ;

		reg_val |= (PLLMODE | ((PREDIV&0x7)<<16) | ((POSTDIV1&0xFF)<<8) | ((POSTDIV2&0xF)<<4) | ((OUTDIV&0xF)<<0));

		SCU->PLLCON |= reg_val;	// PLL Configure setting
		
		/* wait over 190us */
		for(i=0;i<PLL_DELAY;i++); 

		SYST_ACCESS_DIS();
		
		/* Check the PLLCON State */
		while((SCU->PLLCON & (1UL<<31)) == RESET)
		{		
			/* Time out for HSE start up, in 1ms */
			if(StartUpCounter++ > PLL_STARTUP_TIMEOUT)
			{
				 return ERROR;
			}	
		}
		return SUCCESS;
	}
}


/*********************************************************************//**
 * @brief 		Get current value of PLL Status.
 * @param[in]	None.
 * @return		Current value of PLL Status register.
 *********************************************************************/
uint32_t HAL_SCU_PLL_GetStatus(void)
{
	return (SCU->PLLCON);
}



/**********************************************************************//**
 * @brief 		Configure peri clock setting
 * @param[in]	mccrnum: select REG number
 *            - MCCR1 : 1, * - MCCR2 : 2
 *            - MCCR3 : 3, * - MCCR4 : 4
 *            - MCCR5 : 5, * - MCCR6 : 6
 * 					- MCCR7 : 7
 * @param[in]	type : 
 *	           WDT_TYPE, SYSTICK_TYPE, WDT_TYPE, MPWM0_TYPE, MPWM1_TYPE, 
 *						TIMER03_TYPE, TIMER47_TYPE, PGA_TYPE, ADC_TYPE,
 *						PGC_TYPE, PGB_TYPE, FRT_TYPE, UART_TYPE
 * @param[in]	clksrc : 
 *						- SCU_MCCR_CSEL_LSI=0,
 *						- SCU_MCCR_CSEL_MCLK=4,	
 *						- SCU_MCCR_CSEL_HSI=5,
 *						- SCU_MCCR_CSEL_HSE=6,
 * 					- SCU_MCCR_CSEL_PLL =7
 * @param[in]	clkdiv : peri clock divider N
 *						- 0x0 ~ 0xFF
 * @return		None
 **********************************************************************/
void HAL_SCU_MiscClockConfig(uint8_t mccrnum, uint8_t type, uint8_t clksrc, uint8_t clkdiv)
{
	uint32_t reg_val;
	
	SYST_ACCESS_EN();
	
	switch(mccrnum){
		case 1:
			reg_val=SCU->MCCR1;
			if (type==SYSTICK_TYPE){
				reg_val &= ~(SCU_MCCR_CDIV_MSK1 | SCU_MCCR_CSEL_MSK1);
				
				reg_val |= (((clksrc&7)<<8)|((clkdiv&0xFF)<<0));
			}
			if (type==WDT_TYPE){
				reg_val &= ~(SCU_MCCR_CDIV_MSK2 | SCU_MCCR_CSEL_MSK2);
				
				reg_val |= (((clksrc&7)<<24)|((clkdiv&0xFF)<<16));
			}
			SCU->MCCR1=reg_val;
			break;
			
		case 2:
			reg_val=SCU->MCCR2;
			if (type==MPWM0_TYPE){
				reg_val &= ~(SCU_MCCR_CDIV_MSK1 | SCU_MCCR_CSEL_MSK1);
				
				reg_val |= (((clksrc&7)<<8)|((clkdiv&0xFF)<<0));
			}
			if (type==MPWM1_TYPE){
				reg_val &= ~(SCU_MCCR_CDIV_MSK2 | SCU_MCCR_CSEL_MSK2);
				
				reg_val |= (((clksrc&7)<<24)|((clkdiv&0xFF)<<16));
			}
			SCU->MCCR2=reg_val;
			break;
			
		case 3:
			reg_val=SCU->MCCR3;
			if (type==TIMER03_TYPE){
				reg_val &= ~(SCU_MCCR_CDIV_MSK1 | SCU_MCCR_CSEL_MSK1);
				
				reg_val |= (((clksrc&7)<<8)|((clkdiv&0xFF)<<0));
			}
			if (type==TIMER47_TYPE){
				reg_val &= ~(SCU_MCCR_CDIV_MSK2 | SCU_MCCR_CSEL_MSK2);
				
				reg_val |= (((clksrc&7)<<24)|((clkdiv&0xFF)<<16));
			}
			SCU->MCCR3=reg_val;
			break;
			
		case 4:
			reg_val=SCU->MCCR4;
			if (type==ADC_TYPE){
				reg_val &= ~(SCU_MCCR_CDIV_MSK1 | SCU_MCCR_CSEL_MSK1);
				
				reg_val |= (((clksrc&7)<<8)|((clkdiv&0xFF)<<0));
			}
			if (type==PGA_TYPE){
				reg_val &= ~(SCU_MCCR_CDIV_MSK2 | SCU_MCCR_CSEL_MSK2);
				
				reg_val |= (((clksrc&7)<<24)|((clkdiv&0xFF)<<16));
			}
			SCU->MCCR4=reg_val;
			break;
			
		case 5:
			reg_val=SCU->MCCR5;
			if (type==PGB_TYPE){
				reg_val &= ~(SCU_MCCR_CDIV_MSK1 | SCU_MCCR_CSEL_MSK1);
				
				reg_val |= (((clksrc&7)<<8)|((clkdiv&0xFF)<<0));
			}
			if (type==PGC_TYPE){
				reg_val &= ~(SCU_MCCR_CDIV_MSK2 | SCU_MCCR_CSEL_MSK2);
				
				reg_val |= (((clksrc&7)<<24)|((clkdiv&0xFF)<<16));
			}
			SCU->MCCR5=reg_val;
			break;

		case 6:
			reg_val=SCU->MCCR6;
			if (type==FRT_TYPE){
				reg_val &= ~(SCU_MCCR_CDIV_MSK1 | SCU_MCCR_CSEL_MSK1);
				
				reg_val |= (((clksrc&7)<<8)|((clkdiv&0xFF)<<0));
			}
			SCU->MCCR6=reg_val;		
			break;	

		case 7:
			reg_val=SCU->MCCR7;
			if (type==UART_TYPE){
				reg_val &= ~(SCU_MCCR_CDIV_MSK2 | SCU_MCCR_CSEL_MSK2);
				
				reg_val |= (((clksrc&7)<<24)|((clkdiv&0xFF)<<16));
			}
			SCU->MCCR7=reg_val;	
			break;			
	}
	
	SYST_ACCESS_DIS();
}


/**********************************************************************//**
 * @brief 		LVI Interuupt Flag Clear  
 * @param[in]	None
 * @return		None
 **********************************************************************/
void HAL_SCU_ClearLVIIntTFlg(void)
{
	SYST_ACCESS_EN();
	
	SCU->LVISR |= ((0x7A<<24)|(1<<5));		//LVI Flag Clear

	SYST_ACCESS_DIS();
}


/*********************************************************************//**
 * @brief 		Get current value of Clock Monitoring Status.
 * @param[in]	None.
 * @return		Current value of Clock Monitoring Status register.
 *********************************************************************/
FlagStatus HAL_SCU_GetLVIIntStatus(void)
{
	if(SCU->LVISR&(1<<0)){
		return SET;
	}	
	else
	{
		return RESET;
	}
}


/* --------------------------------- End Of File ------------------------------ */
