/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for ADC firmware library
* @version	1.00
* @date		
* @author	ABOV PTS team
*
* Copyright(C) 2020, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void DEBUG_MenuPrint(void);
void DEBUG_Init(void);
void GPIO_Configure(void);
void ADC_Configure(void);
void TIMER_Configure(void);
void ADC_SeqRun(void);
void ADC0_IRQHandler_IT(void);
void TIMER0_IRQHandler_IT(void);
void mainloop(void);
int main (void);
void Error_Handler(void);

/* Private variables ---------------------------------------------------------*/
static ADC_CFG_Type AD_config;
static uint32_t fflag;

static const uint8_t menu[] =
"************************************************\n\r"
" A33M11x_TEST_EXAMPLE \n\r"
"\t - MCU: A33M11x \n\r"
"\t - Core: ARM Cortex-M3 \n\r"
"\t - Communicate via: UART0 - 38400 bps \n\r"
"\t Using ADC in sequence mode \n\r"
"************************************************\n\r";

static TIMER_PERIODICCFG_Type TIMER0_Config;

/* Public Function ------------------------------------------------------------*/
/**********************************************************************
 * @brief		 menu Print
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_MenuPrint(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}

/*******************************************************************************
* @brief      ADC0 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void ADC0_IRQHandler_IT(void)
{
	uint16_t status;
	status = HAL_ADC_GetStatus(ADC0);

	if ((status & ADC_STAT_SEQ)==ADC_STAT_SEQ){	// Seqeunce End Interrupt Flag Check
		HAL_ADC_ClearStatus(ADC0, ADC_STAT_SEQ);
		fflag=1;	
	}
	if ((status & ADC_STAT_TRIGGER)==ADC_STAT_TRIGGER){	// ADC Trigger Interrupt Flag Check
		HAL_ADC_ClearStatus(ADC0, ADC_STAT_TRIGGER);	
		PB->ODR ^= 0x01;
	}
}

/*******************************************************************************
* @brief      Timer0 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void TIMER0_IRQHandler_IT(void)
{
	if ((HAL_TIMER_GetStatus(TIMER0) & TIMER_SR_MFB) == TIMER_SR_MFB){
		HAL_TIMER_ClearStatus(TIMER0, TIMER_SR_MFB);
	}
}

/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	// Toggle Port Setting
	HAL_GPIO_ConfigureFunction(PB, 0, PB0_MUX_PB0);
	HAL_GPIO_ConfigOutput(PB, 0, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PB, 0, PULL_UP_DOWN_DISABLE);
	
	HAL_GPIO_ConfigureFunction(PB, 1, PB1_MUX_PB1);
	HAL_GPIO_ConfigOutput(PB, 1, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PB, 1, PULL_UP_DOWN_DISABLE);	
	
	//	Set ADC input pin (AN0-AN3)
	HAL_GPIO_ConfigureFunction(PA, 0, PA0_MUX_AN0);
	HAL_GPIO_ConfigOutput(PA, 0, INPUT);
	HAL_GPIO_ConfigPullup(PA, 0, PULL_UP_DOWN_DISABLE);	
	
	HAL_GPIO_ConfigureFunction(PA, 1, PA1_MUX_AN1);
	HAL_GPIO_ConfigOutput(PA, 1, INPUT);
	HAL_GPIO_ConfigPullup(PA, 1, PULL_UP_DOWN_DISABLE);	
	
	HAL_GPIO_ConfigureFunction(PA, 2, PA2_MUX_AN2);
	HAL_GPIO_ConfigOutput(PA, 2, INPUT);
	HAL_GPIO_ConfigPullup(PA, 2, PULL_UP_DOWN_DISABLE);	
	
	HAL_GPIO_ConfigureFunction(PA, 3, PA3_MUX_AN3);
	HAL_GPIO_ConfigOutput(PA, 3, INPUT);
	HAL_GPIO_ConfigPullup(PA, 3, PULL_UP_DOWN_DISABLE);	
	
}

/**********************************************************************
 * @brief		TIMER_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void TIMER_Configure(void)
{	
	//------------------------------------------------------
	// TIMER 0 Setting
	//------------------------------------------------------

	TIMER0_Config.CkSel = PCLK_2; // SystemPeriClock=8Mhz=PCLK  CLOCK_IN = PCLK/2=4Mhz 
	TIMER0_Config.Prescaler = 3;    // TCLK = (CLOCK_IN / (PRS+1))  = 4Mhz / 4 = 1Mhz ->1us

	TIMER0_Config.GRA = (500); 	// adc trigger signal = 500us
	TIMER0_Config.GRB = (500); // 			
	TIMER0_Config.Cnt=0;		
	TIMER0_Config.StartLevel=START_LOW;
	TIMER0_Config.AdcTrgEn=ENABLE;
	
	HAL_TIMER_Init(TIMER0, PERIODIC_MODE, &TIMER0_Config); // timer0 setting 
	HAL_TIMER_ConfigInterrupt(TIMER0, TIMER_INTCFG_MBIE, ENABLE);	// timer0 match B interrupt enable
	
	// Enable Interrupt for TIMER0 channel 
	NVIC_SetPriority(TIMER0_IRQn, ((0x01<<1)|0x01));
	NVIC_EnableIRQ(TIMER0_IRQn);	
	
	HAL_TIMER_Cmd(TIMER0, ENABLE); // timer start	
}
/**********************************************************************
 * @brief		ADC_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void ADC_Configure(void)
{	

	// ADC Configuration - Seqeunce Mode-4 Sequence, ADC Clock : 32/8=4MHz, Timer Trigger Mode
	AD_config.Mode = ADC_SINGLE_MODE;
	AD_config.SamplingTime=0x1f;	// Conversion Speed = (Sampling Time + 12 Cycle + 4 Cycle)
	AD_config.SeqCnt = 4;
	AD_config.RestartEn = 0;
	AD_config.TrgSel = ADC_TRIGGER_TIMER;
	
	AD_config.UseClk = ADC_EXTERNAL_CLK;
	AD_config.InClkDiv = 1; 
	
	// Set ADC External Clock = 32MHz/8 = 4MHz
	HAL_SCU_MiscClockConfig(4, ADC_TYPE, SCU_MCCR_CSEL_HSI, 8);
	
	if( HAL_ADC_Init(ADC0, &AD_config) != HAL_OK)
	{
		/* Initialization Error */
     Error_Handler();
	}
	
	HAL_ADC_TriggerlSel(ADC0, 0x00000000); //trigger timer0
	
	HAL_ADC_ChannelSel(ADC0, 0x03020100);	//ch0, ch1, ch2, ch3
	
	HAL_ADC_ClearStatus(ADC0, 0xff); // clear status
	
	HAL_ADC_ConfigInterrupt(ADC0, (ADC_INTEN_SEQ | ADC_INTEN_TRIGGER), ENABLE);	// Trigger Interrupt Enable
	
	// Enable Interrupt for ADC0 channel 
	NVIC_SetPriority(ADC0_IRQn, 3);
	NVIC_EnableIRQ(ADC0_IRQn);
}

/**********************************************************************
 * @brief		ADC_SeqRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void ADC_SeqRun(void)
{	
	uint32_t tmp;
	
	if (AD_config.RestartEn==0) {
			HAL_ADC_Start(ADC0); // start 	
		}
		
		fflag=0;
		while(fflag==0){}

#ifdef _DEBUG_MSG
		_DBG("AN0 = "); _DBH16(ADC0->DR0>>4); _DBG(" ");
		_DBG("AN1 = "); _DBH16(ADC0->DR1>>4); _DBG(" ");
		_DBG("AN2 = "); _DBH16(ADC0->DR2>>4); _DBG(" ");
		_DBG("AN3 = "); _DBH16(ADC0->DR3>>4); _DBG_(" ");			
#endif
			
		for(tmp = 0; tmp < 0x3FFFF; tmp++);
}


/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	/*Configure menu prinf*/
	DEBUG_MenuPrint();

	/*Configure port peripheral*/
	GPIO_Configure();

	/*ADC Configure*/
	ADC_Configure(); 

	/*TIMER Configure*/
	TIMER_Configure(); 
  
	 /* Enable IRQ Interrupts */
	__enable_irq();
	
	while(1)
	{
		/*ADC start*/
		ADC_SeqRun();
	}
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{

	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSE 8 MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing UART port  */
	DEBUG_Init();		
	
	/* Infinite loop */
	mainloop();  


	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
    while (1)
    {
    }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif

/* --------------------------------- End Of File ------------------------------ */
