/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for ADC firmware library
* @version	1.00
* @date		
* @author	ABOV PTS team
*
* Copyright(C) 2020, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void DEBUG_MenuPrint(void);
void DEBUG_Init(void);
void UART_DebugMessage(void);
void GPIO_Configure(void);
void OPAMP_Configure( void);
void ADC_Configure(void);
void AFE_OpampRun(void);
void mainloop(void);
int main (void);
void Error_Handler(void);

/* Private variables ---------------------------------------------------------*/
static const uint8_t menu[] =
"************************************************\n\r"
"\t PGA demo \n\r"
"\t - MCU: A33M11x \n\r"
"\t - Core: ARM Cortex-M3 \n\r"
"\t - Communicate via: UART0 - 38400 bps \n\r"
"\t OPAMP Test Example \n\r"
"\t - INPUT PORT : PA2(AIP0), PA1(AIN0) \n\r"
"\t - OUTPUT PORT : PA0(AO0), CH18 \n\r"
"************************************************\n\r";

static uint16_t adcval0, adcval1, adcval2, adcval3;
static ADC_CFG_Type AD_config;
/* Public Function ------------------------------------------------------------*/
/**********************************************************************
 * @brief		 menu Print
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_MenuPrint(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}


/**********************************************************************
 * @brief		UART_DebugMessage
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void UART_DebugMessage(void)
{
	#ifdef _DEBUG_MSG
		_DBG("(AO0) = "); _DBH16(adcval0); _DBG(" ");
		_DBG("(AIN0) = "); _DBH16(adcval1); _DBG(" ");
		_DBG("(AIP0) = "); _DBH16(adcval2); _DBG(" ");
		_DBG("(Opamp Out0) = "); _DBH16(adcval3); _DBG_(" ");	
		_DBG("\r\n");
	#endif

}

/*******************************************************************************
* @brief      Average of N Data
* @param      val : Pointer contains N data
* @return     Average value
*******************************************************************************/
uint16_t Average8Data(uint16_t *val)
{
	uint32_t i;
	uint32_t adcavg;
	
	adcavg=0;		
	
	for (i=0;i<8;i++){	
		adcavg+=val[i];
	}
	adcavg>>=3;
	
	return (adcavg);
}

/**********************************************************************
 * @brief		PCU_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	// PA0,1,2 OPAMP0 pin setting
	// AO0(PA0)
	HAL_GPIO_ConfigureFunction(PA, 0, PA0_MUX_AO0);
	HAL_GPIO_ConfigOutput(PA, 0, PUSH_PULL_OUTPUT);
	// AIN0(PA1)
	HAL_GPIO_ConfigureFunction(PA, 1, PA1_MUX_AIN0);
	HAL_GPIO_ConfigOutput(PA, 1, INPUT);
	HAL_GPIO_ConfigPullup(PA, 1, PULL_UP_DOWN_DISABLE);	
	// AIP0(PA2)	
	HAL_GPIO_ConfigureFunction(PA, 2, PA2_MUX_AIP0);
	HAL_GPIO_ConfigOutput(PA, 2, INPUT);
	HAL_GPIO_ConfigPullup(PA, 2, PULL_UP_DOWN_DISABLE);
}

/**********************************************************************
 * @brief		OPAMP_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void OPAMP_Configure(void)
{
	HAL_AFE_ClockInit(ENABLE); // opamp clock enable
	
	HAL_AFE_OPAMP_CurrentSet(0, AMPISEL_0PERCENT); // opamp0, Current 0% Setting
	
	HAL_AFE_OPAMP_Cmd(0, ENABLE); // opamp0 opearation enable
}

/**********************************************************************
 * @brief		ADC_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void ADC_Configure(void)
{	
	// ADC Configuration - Single Mode, ADC Clock : 32/8=4MHz, Event Trigger Disable
	AD_config.Mode = ADC_BURST_MODE;;
	AD_config.SamplingTime = 15;
	AD_config.SeqCnt = 4;
	AD_config.RestartEn = 0;
	AD_config.TrgSel = ADC_TRIGGER_DISABLE;

	AD_config.UseClk = ADC_EXTERNAL_CLK;
	AD_config.InClkDiv = 0; 

	HAL_SCU_MiscClockConfig(4, ADC_TYPE, SCU_MCCR_CSEL_HSI, 8);
	
	if( HAL_ADC_Init(ADC0, &AD_config) != HAL_OK)
	{
		/* Initialization Error */
     Error_Handler();
	}
	
	HAL_ADC_ChannelSel_1(ADC0, 0x12020100);	//ch0-AN0, ch1-AN1, ch2-AN2, ch3-OpampO
	
}

/**********************************************************************
 * @brief		AFE_OpampRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void AFE_OpampRun(void)
{	
	
	HAL_ADC_Start(ADC0); //adc start 	
	while((HAL_ADC_GetStatus(ADC0) & (ADC_STAT_SEQ)) !=(ADC_STAT_SEQ)){}	// ADC Seqeunce Conversion End Flag Check					
	HAL_ADC_ClearStatus(ADC0, ADC_STAT_SEQ);	// Seqeunce Conversion End Flag Clear
	
	adcval0 = (ADC0->DR0);	// ch0(AO0) get data
	adcval1 = (ADC0->DR1);	// ch1(AIN0) get data
	adcval2 = (ADC0->DR2);	// ch2(AIP0) get data
	adcval3 = (ADC0->DR3);	// ch3(OPAMP0O) get data
		
	UART_DebugMessage();
}

/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	/*Configure menu prinf*/
	DEBUG_MenuPrint();

	/*Configure port peripheral*/
	GPIO_Configure();

	/*OPAMP Configure*/
	OPAMP_Configure();

	/*ADC Configure*/
	ADC_Configure(); 
  
	 /* Enable IRQ Interrupts */
	__enable_irq();
	
	while(1)
	{
		/* AFE Opamp Run  */
	   AFE_OpampRun(); 
	}
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{

	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSE 8 MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing UART port  */
	DEBUG_Init();		
	
	/* Infinite loop */
	mainloop();  


	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
    while (1)
    {
    }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif

/* --------------------------------- End Of File ------------------------------ */
