/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for ADC firmware library
* @version	1.00
* @date		
* @author	ABOV PTS team
*
* Copyright(C) 2020, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void GPIOA_IRQHandler_IT(void);
void DEBUG_MenuPrint(void);
void DEBUG_Init(void);
void GPIO_ExtIntRun(void);
void mainloop(void);
int main (void);
void Error_Handler(void);

/* Private variables ---------------------------------------------------------*/
static const uint8_t menu[] =
"************************************************\n\r"
" GPIO demo \n\r"
"\t - MCU: A33M11x \n\r"
"\t - Core: ARM Cortex-M3 \n\r"
"\t - Communicate via: UART0 - 38400 bps \n\r"
" This example used to configure the\n\r "
" GPIO PA0 as source of external Interrupt \n\r"
"************************************************\n\r";
static uint8_t mode_level[]=
	"\t - 0: Prohibit external Interrupt \n\r"
	"\t - 1: low level Interrupt \n\r"
	"\t - 2: high level Interrupt \n\r";
static uint8_t mode_edge[]=
	"\t - 0: Prohibit external Interrupt \n\r"
	"\t - 1: falling edge Interrupt \n\r"
	"\t - 2: rising edge Interrupt \n\r"
	"\t - 3: both edge Interrupt \n\r";
	
static uint8_t flag;
static uint8_t val_level_or_edge, mode;

/* Public Function ------------------------------------------------------------*/
/*******************************************************************************
* @brief      GPIO A interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void GPIOA_IRQHandler_IT(void)
{
	uint32_t status;
	
	status = HAL_GPIO_EXTI_GetStatus(PA);
	
	if(status & (3UL<<(0<<1)))	// PA0 Interrupt Flag Check
	{
		HAL_GPIO_EXTI_ClearPin(PA, status & (3UL<<(0<<1)));
		_DBG("\r\nPA0 Interrupt ok!!\r\n");
	}			
}

/**********************************************************************
 * @brief		 menu Print
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_MenuPrint(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}

/**********************************************************************
 * @brief		PCU_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	// external interrupt test pin PA0
	HAL_GPIO_ConfigureFunction(PA, 0, INPUT);
	HAL_GPIO_ConfigPullup(PA, 0, PULL_UP_ENABLE);
	HAL_GPIO_SetDebouncePin(PA, 0, ENABLE);
	
	// Enable Interrupt for GPIOA channel 
	NVIC_SetPriority(GPIOA_IRQn, 3);	
	NVIC_EnableIRQ(GPIOA_IRQn);
	
	// port A (PA, PB) debounce clock setting
	HAL_SCU_MiscClockConfig(4,PGA_TYPE,SCU_MCCR_CSEL_MCLK,100);		//LSI=500KHz, N=100, 500KHz/100=5KHz, 0.2ms
	
}

/**********************************************************************
 * @brief		GPIO_ExtIntRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_ExtIntRun(void)
{		
		/* Get some inputs */
	_DBG("Please input character 1:LEVEL(Non-pending), 2:LEVEL(Pending), 3: EDGE! \n\r");	
	
	val_level_or_edge = 0;
	flag = 1;
	while (flag)
	{	
		val_level_or_edge = _DG;
		switch (val_level_or_edge)
		{
			case '1':
				_DBC(val_level_or_edge);
				val_level_or_edge = IER_LEVEL_NON_PENDING;
				flag = 0; 
				break;			
			case '2': 
				_DBC(val_level_or_edge);
				val_level_or_edge = IER_LEVEL_PENDING;
				flag = 0; 
				break;
			case '3':
				_DBC(val_level_or_edge);
				val_level_or_edge = IER_EDGE;
				flag = 0; 
				break;
			default:
				_DBG("\n\r...Please input character 1:LEVEL(Non-pending), 2:LEVEL(Pending), 3: EDGE!  \n\r");
				break;
		}
	}

	/* edge or level mode selection */
	if (val_level_or_edge==IER_LEVEL_NON_PENDING||val_level_or_edge==IER_LEVEL_PENDING){
		_DBG(mode_level);
	}
	else {
		_DBG(mode_edge);		
	}
	
	mode = 0;
	flag = 1;
	while (flag)
	{
		mode = _DG;
		if (val_level_or_edge==IER_LEVEL_NON_PENDING){
			switch (mode)
			{
				case '0':
				case '1': 
				case '2': 					
					_DBC(mode);
					mode = mode -0x30;
					flag = 0; 
					break;
				default:
					_DBG("\n\r...Please input digit from 0 to 2 only! \n\r");
					break;
			}			
		}
		else {
			switch (mode)
			{
				case '0':
				case '1': 
				case '2': 		
				case '3': 						
					_DBC(mode);
					mode = mode -0x30;
					flag = 0; 
					break;
				default:
					_DBG("\n\r...Please input digit from 0 to 3 only! \n\r");
					break;
			}			
		}
	}

	HAL_GPIO_EXTI_Config(PA, 0, val_level_or_edge, mode);
}


/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	/*Configure menu prinf*/
	DEBUG_MenuPrint();
 
	/*Configure port peripheral*/
	GPIO_Configure();
	
	/* Enable IRQ Interrupts */
	__enable_irq();
	
		/*GPIO ExtInt Test start*/
		GPIO_ExtIntRun();
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{

	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSE 8 MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing UART port  */
	DEBUG_Init();		
	
	/* Infinite loop */
	mainloop();  


	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
    while (1)
    {
    }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif

/* --------------------------------- End Of File ------------------------------ */
