/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for ADC firmware library
* @version	1.00
* @date		
* @author	ABOV PTS team
*
* Copyright(C) 2020, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void DEBUG_MenuPrint(void);
void DEBUG_Init(void);
void GPIO_Configure(void);
void SYSTICK_Configure(void);
void PWR_DeepSleepRun(void);
void mainloop(void);
int main (void);
void Error_Handler(void);
void SysTick_Handler_IT(void);
void GPIOB_IRQHandler_IT(void);

/* Private variables ---------------------------------------------------------*/
static const uint8_t menu[] =
"************************************************\n\r"
" Deep Sleep demo \n\r"
"\t - MCU: A33M11x \n\r"
"\t - Core: ARM Cortex-M3 \n\r"
"\t - Communicate via: UART0 - 38400 bps \n\r"
"************************************************\n\r";

static uint32_t MilliSec;
	
/* Public Function ------------------------------------------------------------*/
/*******************************************************************************
* @brief      SysTick interrupt handler @ 1000Hz(1ms)
* @param      None
* @return     None
*******************************************************************************/
void SysTick_Handler_IT(void) {
	if (MilliSec) {
		MilliSec--;
	}
}

/*******************************************************************************
* @brief      GPIO B interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void GPIOB_IRQHandler_IT(void)
{
	uint32_t status;
	
	status = HAL_GPIO_EXTI_GetStatus(PB);
	if (status & (3<<(1<<1)))	// PB1 Interrupt Flag Check
	{
		HAL_GPIO_EXTI_ClearPin(PB, status&(3<<(1<<1)));
	}
}

/**********************************************************************
 * @brief		 menu Print
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_MenuPrint(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}


/**********************************************************************
 * @brief		PCU_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	// Test Pin setting PB0
	HAL_GPIO_ConfigureFunction(PB, 0, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PB, 0, PULL_UP_DOWN_DISABLE);	
	HAL_GPIO_EXTI_ClearPin(PB, _BIT(0));
	
	// External Interrupt Test pin PB1
	HAL_GPIO_ConfigOutput(PB, 1, INPUT);
	HAL_GPIO_ConfigPullup(PB, 1, PULL_UP_ENABLE);
	
	// PB1 External Interrupt : Falling Edge
	HAL_GPIO_EXTI_Config(PB, 1, IER_EDGE, ICR_FALLING_EDGE_INT);
	
	// Enable Interrupt for GPIOB channel 
	NVIC_SetPriority(GPIOB_IRQn, 7);
	NVIC_EnableIRQ(GPIOB_IRQn);
	
	// External Input Test pin PB2
	HAL_GPIO_ConfigureFunction(PB, 2, INPUT);
	HAL_GPIO_ConfigPullup(PB, 2, PULL_DOWN_ENABLE);

	// PB Wakeup soure Enable
	HAL_SCU_WakeUpSRCCmd(GPIOBWUE, ENABLE);
}

/**********************************************************************
 * @brief		SYSTICK_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void SYSTICK_Configure(void)
{	
	//1msec interrupt 
	SysTick_Config(SystemCoreClock/1000);  
}

/**********************************************************************
 * @brief		PWR_DeepSleepRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void PWR_DeepSleepRun(void)
{	
_DBG("Press PB2 to VDD enter deep sleep mode !");
	while((HAL_GPIO_ReadPin(PB) & (1UL<<2)) != (1UL<<2));
		
	_DBG("\n\rEnter Deep Sleep... Connect PB1 pin to GND to exit sleep mode !  ");
	
	//System Clock : LSI Setting - 500kHz
	HAL_SCU_SystemClockChange(SCU_SCCR_MCLKSEL_LSI);

	// Except LSI, Clock Disable
	HAL_SCU_ClockSRCCmd((SCU_CSCR_HSE_ON|SCU_CSCR_HSI_ON), DISABLE);
	
	// Run PowerDown Mode 
	HAL_PWR_EnterPowerDownMode();
	
	//LSI, HSE, HSI Clock Enable
	HAL_SCU_ClockSRCCmd((SCU_CSCR_LSI_ON|SCU_CSCR_HSE_ON|SCU_CSCR_HSI_ON), ENABLE);
	
	//clock stability time
	MilliSec = 100;
	while(MilliSec);
	
	// System Clock : HSE Setting (8MHz)
	HAL_SCU_SystemClockChange(SCU_SCCR_MCLKSEL_HSE);
	
	_DBG("\n\rWaked Up from Deep sleep mode!");
	
	while(1)
	{
		MilliSec = 1000;
		while(MilliSec);
		
		HAL_GPIO_SetPin(PB, _BIT(0));
		
		MilliSec = 1000;
		while(MilliSec);
		
		HAL_GPIO_ClearPin(PB, _BIT(0));
	}
}


/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	/*Configure menu prinf*/
	DEBUG_MenuPrint();
 
	/*Configure port peripheral*/
	GPIO_Configure();

	/*SYSTICK Configure*/
	SYSTICK_Configure(); 
  
	/* Enable IRQ Interrupts */
	__enable_irq();
	
	while(1)
	{
		/*PWR Deep Sleep Test start*/
		PWR_DeepSleepRun();
	}
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{

	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSE 8 MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing UART port  */
	DEBUG_Init();		
	
	/* Infinite loop */
	mainloop();  


	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
    while (1)
    {
    }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif

/* --------------------------------- End Of File ------------------------------ */
