/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for ADC firmware library
* @version	1.00
* @date		
* @author	ABOV PTS team
*
* Copyright(C) 2020, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void DEBUG_MenuPrint(void);
void DEBUG_Init(void);
void GPIO_Configure(void);
void SYSTICK_Configure(void);
void PWR_StopWakeupFrtRun(void);
void mainloop(void);
int main (void);
void Error_Handler(void);
void SysTick_Handler_IT(void);
void WDT_IRQHandler_IT(void);

/* Private variables ---------------------------------------------------------*/
static const uint8_t menu[] =
"************************************************\n\r"
" Example of wake-up using frt in sleep mode \n\r"
"\t - MCU: A33M11x \n\r"
"\t - Core: ARM Cortex-M3 \n\r"
"\t - Communicate via: UART0 - 38400 bps \n\r"
"\t - FRT Match Mode \n\r"
"\t  LSI500KHz, FCNT:500000(1s) \n\r"
"************************************************\n\r";

static uint32_t MilliSec;
static uint8_t wdtf;

/* Public Function ------------------------------------------------------------*/
/*******************************************************************************
* @brief      WDT interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void WDT_IRQHandler_IT(void) {
	HAL_WDT_UpdateTimeOut(500000);	//1s
	
	if(wdtf == 0){
		wdtf=1;
		HAL_GPIO_ClearPin(PB,_BIT(0));
	}
	else{
		wdtf=0;
		HAL_GPIO_SetPin(PB,_BIT(0));
	}
}


/*******************************************************************************
* @brief      SysTick interrupt handler @ 1000Hz(1ms)
* @param      None
* @return     None
*******************************************************************************/
void SysTick_Handler_IT(void) {
	if (MilliSec) {
		MilliSec--;
	}
}


/**********************************************************************
 * @brief		 menu Print
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_MenuPrint(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}


/**********************************************************************
 * @brief		PCU_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	// Test Pin setting PB0
	HAL_GPIO_ConfigureFunction(PB, 0, PB0_MUX_PB0);
	HAL_GPIO_ConfigOutput(PB, 0, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PB, 0, PULL_UP_DOWN_DISABLE);	
	HAL_GPIO_EXTI_ClearPin(PB, _BIT(0));
}


/**********************************************************************
 * @brief		WDT_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void WDT_Configure(void)
{
	WDT_CFG_Type wdtCfg;
	
	// WDT initialization
	HAL_WDT_Init();
	
	// WDT MCCR1=LSI500KHz
	HAL_SCU_MiscClockConfig(1, WDT_TYPE, SCU_MCCR_CSEL_LSI, 1);
	
	wdtCfg.wdtResetEn = DISABLE;
	wdtCfg.wdtCountEn = ENABLE;
	wdtCfg.wdtDbgEn = ENABLE;
	wdtCfg.wdtClkSel = SET; //SET:external from MCCR1 , RESET:PCLK
	wdtCfg.wdtPrescaler = WDT_DIV_1;  //WDTCLK = WDTCLKIN / PRS
	wdtCfg.wdtTmrConst = 500000; // 500KHz/500000 = 1s /
	
	HAL_WDT_Configure(wdtCfg);
	
	/* preemption = 1, sub-priority = 1 */
  NVIC_SetPriority(WDT_IRQn, ((0x01<<1)|0x01));
	/* Enable Interrupt for TIMER0 channel */
  NVIC_EnableIRQ(WDT_IRQn);

	// WDT Start
	HAL_WDT_Start(ENABLE);
}

/**********************************************************************
 * @brief		SYSTICK_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void SYSTICK_Configure(void)
{	
	//1msec interrupt 
	SysTick_Config(SystemCoreClock/1000);  
}

/**********************************************************************
 * @brief		PWR_StopWakeupWdtRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void PWR_StopWakeupWdtRun(void)
{
	// LSI500KHz, Internal VDC 1.5V enable
	HAL_SCU_SMRCmd((SCU_SMR_LSIAON|SCU_SMR_VDCAON),ENABLE);
	
	// Wdt wake-up source enable
	HAL_SCU_WakeUpSRCCmd(WDTWUE,ENABLE);
	
	_DBG("\n\rEnter Stop... !  ");
	
	// Enter stop mode
	HAL_PWR_EnterPowerDownMode();
	
	/*SYSTICK Configure*/
	SYSTICK_Configure(); 
	
	//clock stability time
	MilliSec = 100;
	while(MilliSec);
	
	// System clock initialization
	SystemClock_Config();
	
	_DBG("\n\rWaked Up from stop mode!");
	
	while(1);
}


/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	/*Configure menu prinf*/
	DEBUG_MenuPrint();
 
	/*Configure port peripheral*/
	GPIO_Configure();

	/*WDT Configure*/
	WDT_Configure(); 
	  	
	/* Enable IRQ Interrupts */
	__enable_irq();

	/*PWR stop wake-up Wdt test start*/
	PWR_StopWakeupWdtRun();
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{

	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSE 8 MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing UART port  */
	DEBUG_Init();		
	
	/* Infinite loop */
	mainloop();  


	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
    while (1)
    {
    }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif

/* --------------------------------- End Of File ------------------------------ */
